#ifdef MASTERCAM_AFX
#include "stdafx.h"
#endif

#include "FeatureMaker.h"

// Calls to this function can be eliminated from code by unsetting #MY_TRACE in FeatureMaker.h
void featuremaker_function_trace(const char * fmt, ...)
{
#ifdef MY_TRACE
    if (pfLog == NULL)
	return;
    va_list ap;
    va_start(ap, fmt);

    char buff[10240];
    char * c = buff;
    vsprintf (c, fmt, ap); 

    va_end(ap);

    static int count = 0;
//    printf ("Feature_maker\tCall#%d: %s\n", count, message);
    fprintf (pfLog, "Feature_maker\tCall#%d: %s\n", count, buff);
    count++;   
#endif
}

// Error reports can go to a message box, the trace file or nowhere depending on setting of #MY_REPORT and #MY_TRACE
void featuremaker_report_error(const char * fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);

    char buff[10240];
    char * c = buff;
    vsprintf (c, fmt, ap);

    va_end(ap);

	// Windows gadget to show a warning box requires windows.h
#ifdef MY_REPORT
    MessageBox (NULL, buff, "WARNING", MB_ICONSTOP);
#endif
#ifdef MY_TRACE
    if (pfLog == NULL)
	return;
    fprintf (pfLog, "Feature_maker\tERROR: %s\n", buff);
#endif
}

StixMtrx new_placement (double x, double y, double z, double i, double j, double k, double a, double b, double c) 
{
    TRACE_CALL ("New placement at (%g, %g, %g) z_axis = (%g, %g, %g) x_axis = (%g, %g, %g) ", x, y, z, i, j, k, a, b, c);

    StixMtrx M;

    M.w() = StixVctr (z, y, z);
    M.z() = StixVctr (i, j, k);
    M.x() = StixVctr (a, b, c);
    M.y() = M.z() * M.x();

    return M;
}

StixMtrx new_simple_placement (double x, double y, double z) 
{
    TRACE_CALL ("New simple placement at (%g, %g, %g) z_axis = (%g, %g, %g) x_axis = (%g, %g, %g) ", x, y, z, 0, 0, 1, 1, 0, 0);

    StixMtrx M;

    M.w() = StixVctr (z, y, z);
    M.z() = StixVctr (0, 0, 1);
    M.x() = StixVctr (1, 0, 0);
    M.y() = M.z() * M.x();

    return M;
}

Toolpath_feature *new_toolpath_feature (Workpiece *piece, char * nme, StixMtrx placement)
{
    if (piece == NULL) {
	REPORT_ERROR ("New toolpath feature: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New toolpath feature for workpiece %s name= %s", piece->get_its_id (), nme);

    RoseDesign *des = piece->getRootObject()->design();

    Toolpath_feature *feature = Toolpath_feature::newInstance (des);
    
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    return feature;
}

// Faces can be added to all the different kinds of features but they are most commonly added to toolpath_features
Manufacturing_feature_IF *add_face_to_feature (Manufacturing_feature_IF *feature, stp_advanced_face *face)
{
    if (feature == NULL || face == NULL) {
	REPORT_ERROR ("Add face to feature: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Add face %s to feature %s", feature->get_its_id (), face->name());

    feature->add_explicit_representation (face);
    return feature;
}

Round_hole *new_round_hole (Workpiece *piece, char * nme, double depth, double diameter, StixMtrx placement, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New round hole: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New round hole for workpiece %s name= %s, depth = %g diameter = %g  units=%s", piece->get_its_id (), nme, depth, diameter, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();

    Round_hole * hole = Round_hole::newInstance (des);
    
    hole->put_feature_placement (placement.makeAp3dIn (des));
    hole->put_its_id (nme);
    hole->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_measure_item (des, depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));

    Circular_closed_profile * profile = Circular_closed_profile::newInstance (des);
    profile->put_diameter (stix_make_measure_item (des, diameter, length_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));

    hole->put_diameter(profile->getRoot());
    hole->put_course_of_travel (path->getRoot());
//    hole->put_bottom_condition ((Through_bottom_condition::newInstance(des))->getRoot());

    return hole;
}

// make the hole bottoms
Round_hole *set_hole_though_bottom (Round_hole *hole)
{
    if (hole == NULL) {
	REPORT_ERROR ("Set hole though bottom: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set bottom for hole %s to through", hole->get_its_id ());

    RoseDesign *des = hole->getRootObject()->design();
    RoseObject *obj = hole->get_bottom_condition();

    if (Through_bottom_condition::find (obj))
	ARMdelete(Through_bottom_condition::find (obj));
    else if (Flat_hole_bottom::find (obj))
	ARMdelete(Flat_hole_bottom::find (obj));
    else if (Spherical_hole_bottom::find (obj))
	ARMdelete(Spherical_hole_bottom::find (obj));
    else if (Conical_hole_bottom::find (obj))
	ARMdelete(Conical_hole_bottom::find (obj));
    else if (Flat_with_radius_hole_bottom::find (obj))
	ARMdelete(Flat_with_radius_hole_bottom::find (obj));

    Through_bottom_condition *thruie = Through_bottom_condition::newInstance (des);

    hole->put_bottom_condition (thruie->getRoot());
    return hole;
}

Round_hole *set_hole_flat_bottom (Round_hole *hole)
{
    if (hole == NULL) {
	REPORT_ERROR ("Set hole flat bottom: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set bottom for hole %s to flat", hole->get_its_id ());

    RoseDesign *des = hole->getRootObject()->design();
    RoseObject *obj = hole->get_bottom_condition();

    if (Through_bottom_condition::find (obj))
	ARMdelete(Through_bottom_condition::find (obj));
    else if (Flat_hole_bottom::find (obj))
	ARMdelete(Flat_hole_bottom::find (obj));
    else if (Spherical_hole_bottom::find (obj))
	ARMdelete(Spherical_hole_bottom::find (obj));
    else if (Conical_hole_bottom::find (obj))
	ARMdelete(Conical_hole_bottom::find (obj));
    else if (Flat_with_radius_hole_bottom::find (obj))
	ARMdelete(Flat_with_radius_hole_bottom::find (obj));

    Flat_hole_bottom * flattie = Flat_hole_bottom::newInstance (des);

    hole->put_bottom_condition (flattie->getRoot());
    return hole;
}

Round_hole *set_hole_conical_bottom (Round_hole * hole, double tip_angle, double tip_radius, StixUnit angle_unit, StixUnit length_unit)
{
    if (hole == NULL || angle_unit == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set hole conical bottom: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set bottom for hole %s to conical with angle = %g %s and radius = %g %s", hole->get_its_id (), tip_angle, stix_get_unit_name (angle_unit), tip_radius, stix_get_unit_name (length_unit));

    RoseDesign *des = hole->getRootObject()->design();
    RoseObject *obj = hole->get_bottom_condition();

    if (Through_bottom_condition::find (obj))
	ARMdelete(Through_bottom_condition::find (obj));
    else if (Flat_hole_bottom::find (obj))
	ARMdelete(Flat_hole_bottom::find (obj));
    else if (Spherical_hole_bottom::find (obj))
	ARMdelete(Spherical_hole_bottom::find (obj));
    else if (Conical_hole_bottom::find (obj))
	ARMdelete(Conical_hole_bottom::find (obj));
    else if (Flat_with_radius_hole_bottom::find (obj))
	ARMdelete(Flat_with_radius_hole_bottom::find (obj));

    Conical_hole_bottom * connie = Conical_hole_bottom::newInstance (des);
    connie->put_tip_angle (stix_make_angle_measure_item (des, tip_angle, angle_unit));
    connie->put_tip_radius (stix_make_length_measure_item (des, tip_radius, length_unit));

    hole->put_bottom_condition (connie->getRoot());
    return hole;
}

Round_hole *set_hole_spherical_bottom (Round_hole *hole, double radius, StixUnit length_unit)
{
    if (hole == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set hole spherical bottom: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set bottom for hole %s to spherical with radius = %g %s", hole->get_its_id (), radius, stix_get_unit_name (length_unit));

    RoseDesign *des = hole->getRootObject()->design();
    RoseObject *obj = hole->get_bottom_condition();

    if (Through_bottom_condition::find (obj))
	ARMdelete(Through_bottom_condition::find (obj));
    else if (Flat_hole_bottom::find (obj))
	ARMdelete(Flat_hole_bottom::find (obj));
    else if (Spherical_hole_bottom::find (obj))
	ARMdelete(Spherical_hole_bottom::find (obj));
    else if (Conical_hole_bottom::find (obj))
	ARMdelete(Conical_hole_bottom::find (obj));
    else if (Flat_with_radius_hole_bottom::find (obj))
	ARMdelete(Flat_with_radius_hole_bottom::find (obj));

    Spherical_hole_bottom * spherie = Spherical_hole_bottom::newInstance (des);
    spherie->put_radius (stix_make_length_measure_item (des, radius, length_unit));

    hole->put_bottom_condition (spherie->getRoot());
    return hole;
}

Round_hole *set_hole_flat_with_radius_bottom (Round_hole *hole, double corner_radius, StixUnit length_unit)
{
    if (hole == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set hole flat with radius bottom: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set bottom for hole %s to flat with radius = %g %s", hole->get_its_id (), corner_radius, stix_get_unit_name (length_unit));

    RoseDesign *des = hole->getRootObject()->design();
    RoseObject *obj = hole->get_bottom_condition();

    if (Through_bottom_condition::find (obj))
	ARMdelete(Through_bottom_condition::find (obj));
    else if (Flat_hole_bottom::find (obj))
	ARMdelete(Flat_hole_bottom::find (obj));
    else if (Spherical_hole_bottom::find (obj))
	ARMdelete(Spherical_hole_bottom::find (obj));
    else if (Conical_hole_bottom::find (obj))
	ARMdelete(Conical_hole_bottom::find (obj));
    else if (Flat_with_radius_hole_bottom::find (obj))
	ARMdelete(Flat_with_radius_hole_bottom::find (obj));

    Flat_with_radius_hole_bottom * flattie = Flat_with_radius_hole_bottom::newInstance (des);
    flattie->put_corner_radius (stix_make_length_measure_item (des, corner_radius, length_unit));

    hole->put_bottom_condition (flattie->getRoot());
    return hole;
}

Round_hole *set_hole_angle_taper (Round_hole *hole, double angle, StixUnit angle_unit)
{
    if (hole == NULL || angle_unit == NULL) {
	REPORT_ERROR ("Set hole angle taper: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set angle taper for hole %s to %g  units=%s", hole->get_its_id (), angle, stix_get_unit_name (angle_unit));

    RoseDesign *des = hole->getRootObject()->design();    

    Angle_taper *taper = Angle_taper::newInstance (des);
    taper->put_angle (stix_make_angle_measure_item (des, angle, angle_unit));

    hole->put_change_in_diameter (taper->getRoot ());
    return hole;
}

Round_hole *set_hole_final_diameter_taper (Round_hole *hole, double final_diameter, StixUnit length_unit)
{
    if (hole == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set hole final diameter taper: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set final diameter taper for hole %s to %g  units=%s", hole->get_its_id (), final_diameter, stix_get_unit_name (length_unit));

    RoseDesign *des = hole->getRootObject()->design();    

    Diameter_taper *taper = Diameter_taper::newInstance (des);
    taper->put_final_diameter (stix_make_angle_measure_item (des, final_diameter, length_unit));

    hole->put_change_in_diameter (taper->getRoot ());
    return hole;
}

Closed_pocket *new_closed_rectangular_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double length, double width, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New rectangular pocket: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New rectangular pocket for workpiece %s name= %s, depth = %g length,width = %g,%g  units=%s", piece->get_its_id (), nme, pocket_depth, length, width, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Closed_pocket * pocket = Closed_pocket::newInstance (des);
    pocket->put_feature_placement (placement.makeAp3dIn (des));
    pocket->put_its_id (nme);
    pocket->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, pocket_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_course_of_travel (path->getRoot());

    Rectangular_closed_profile * profile = Rectangular_closed_profile::newInstance (des);
    profile->put_profile_length (stix_make_length_measure_item (des, length, length_unit));
    profile->put_profile_width (stix_make_length_measure_item (des, width, length_unit));;
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_feature_boundary(profile->getRoot());

//    pocket->put_bottom_condition ((Through_pocket_bottom_condition::newInstance(des))->getRoot());

    return pocket;
}

Closed_pocket *new_closed_circular_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double diameter, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New circular pocket: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New circular pocket for workpiece %s name= %s, depth = %g diameter = %g  units=%s", piece->get_its_id (), nme, pocket_depth, diameter, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Closed_pocket * pocket = Closed_pocket::newInstance (des);
    pocket->put_feature_placement (placement.makeAp3dIn (des));
    pocket->put_its_id (nme);
    pocket->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, pocket_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_course_of_travel (path->getRoot());

    Circular_closed_profile * profile = Circular_closed_profile::newInstance (des);
    profile->put_diameter (stix_make_length_measure_item (des, diameter, length_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_feature_boundary(profile->getRoot());

//    pocket->put_bottom_condition ((Through_pocket_bottom_condition::newInstance(des))->getRoot());

    return pocket;
}

stp_bounded_curve *start_profile_curve (RoseDesign *des, double x, double y, double z)
{
     if (des == NULL) {
	REPORT_ERROR ("Start profile curve: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Starting profile curve for design %s at (%g, %g, %g)",  des->name(), x, y, z);

    stp_polyline *curve = pnewIn(des) stp_polyline;
    curve->name("");
    stp_cartesian_point *first = pnewIn (des) stp_cartesian_point;
    first->name ("");
    first->coordinates()->add (x)->add (y)->add (z);
    curve->points()->add (first);

    return curve;
}

stp_bounded_curve * profile_curve_add_xyz (stp_bounded_curve *curve, double x, double y, double z)
{
     if (curve == NULL) {
	REPORT_ERROR ("Profile curve add xyz: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Profile curve add xyz (%g, %g, %g)",  x, y, z);

    RoseDesign *des = curve->design();
    
    if (!curve->isa (ROSE_DOMAIN (stp_polyline))) {
	REPORT_ERROR ("Profile curve add xyz: not processing composite curves yet");
	return NULL;
    }

    stp_polyline *poly = ROSE_CAST (stp_polyline, curve);
    stp_cartesian_point *next = pnewIn (des) stp_cartesian_point;
    next->name ("");
    next->coordinates()->add (x)->add (y)->add (z);
    poly->points()->add (next);

    return curve;
}

stp_bounded_curve *profile_curve_add_arc (stp_bounded_curve *curve, double x, double y, double z)			    // via point to create an arc from pt[-1] and pt[+1]
{
     if (curve == NULL) {
	REPORT_ERROR ("Profile curve via arc: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Profile curve via arc (%g, %g, %g)",  x, y, z);

    RoseDesign *des = curve->design();

    if (!curve->isa (ROSE_DOMAIN (stp_polyline))) {
	REPORT_ERROR ("Profile curve via arc: not processing composite curves yet");
	return NULL;
    }

    stp_polyline *poly = ROSE_CAST (stp_polyline, curve);
    stp_via_arc_point *next = pnewIn (des) stp_via_arc_point;
    next->name ("via_arc");
    next->coordinates()->add (x)->add (y)->add (z);
    poly->points()->add (next);

    return curve;
}

Closed_pocket *new_closed_path_pocket (Workpiece *piece, char * nme,  StixMtrx placement, stp_bounded_curve *curve, double pocket_depth, StixUnit length_unit)
{
    if (piece == NULL || curve == NULL || length_unit == NULL) {
	REPORT_ERROR ("New closed path pocket: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New closed path pocket for design %s name= %s, depth = %g %s", piece->get_its_id (), nme, pocket_depth, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject ()->design ();

    // Could check the profile
    // If a polyline then more than two points

    Closed_pocket * pocket = Closed_pocket::newInstance (des);
    pocket->put_feature_placement (placement.makeAp3dIn (des));
    pocket->put_its_id (nme);
    pocket->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, pocket_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_course_of_travel (path->getRoot());

    General_closed_profile * profile = General_closed_profile::newInstance (des);
    profile->put_closed_profile_shape (curve); 
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_feature_boundary(profile->getRoot());

//    pocket->put_bottom_condition ((Through_pocket_bottom_condition::newInstance(des))->getRoot());

    return pocket;
}

Pocket_IF *set_pocket_orthogonal_radius (Pocket_IF *pocket, double orthogonal_radius, StixUnit length_unit) 
{
    if (pocket == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set pocket orthogonal radius: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set pocket orthogonal radius for pocket %s to %g  units=%s", pocket->get_its_id (), orthogonal_radius, stix_get_unit_name (length_unit));

    RoseDesign *des = pocket->getRootObject()->design(); 
    pocket->put_orthogonal_radius (stix_make_length_measure_item (des, orthogonal_radius, length_unit));
    return pocket;
}

Pocket_IF *set_pocket_planar_radius (Pocket_IF *pocket, double planar_radius, StixUnit length_unit) 
{
    if (pocket == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set pocket planar radius: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set pocket planar radius for pocket %s to %g  units=%s", pocket->get_its_id (), planar_radius, stix_get_unit_name (length_unit));

    RoseDesign *des = pocket->getRootObject()->design(); 
    Pocket_bottom_condition_IF *bottom = Pocket_bottom_condition_IF::find (pocket->get_bottom_condition());
    if (bottom)
	bottom->put_planar_radius (stix_make_length_measure_item (des, planar_radius, length_unit));
    else {
	REPORT_ERROR ("Set pocket planar radius: pocket must have bottom to set planar radius");
	return NULL;
    }
    return pocket;
}

Pocket_IF *set_pocket_slope (Pocket_IF *pocket, double slope, StixUnit angle_unit) 
{
    if (pocket == NULL || angle_unit == NULL) {
	REPORT_ERROR ("Set pocket slope: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set pocket slope for pocket %s to %g with units=%s", pocket->get_its_id (), slope, stix_get_unit_name (angle_unit));

    RoseDesign *des = pocket->getRootObject()->design(); 
    Angle_taper * taper = Angle_taper::newInstance(des);
    taper->put_angle (stix_make_angle_measure_item (des, slope, angle_unit));
    pocket->put_slope (taper->getRoot());
    return pocket;
}

// the default condition when pockets are created
Pocket_IF *set_pocket_through_bottom (Pocket_IF *pocket)
{
    if (pocket == NULL) {
	REPORT_ERROR ("Set pocket though bottom: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set bottom for pocket %s to through", pocket->get_its_id ());

    RoseDesign *des = pocket->getRootObject()->design();
    RoseObject *obj = pocket->get_bottom_condition();

    if (Through_pocket_bottom_condition::find (obj)) 
	ARMdelete(Through_pocket_bottom_condition::find (obj));
    else if (Planar_pocket_bottom_condition::find (obj)) 
	ARMdelete(Planar_pocket_bottom_condition::find (obj));
    else if (Radiused_pocket_bottom_condition::find (obj)) 
	ARMdelete(Radiused_pocket_bottom_condition::find (obj));

    Through_pocket_bottom_condition *thruie = Through_pocket_bottom_condition::newInstance (des);
    pocket->put_bottom_condition (thruie->getRoot());
    return pocket;
}

Pocket_IF *set_pocket_flat_bottom (Pocket_IF *pocket)
{
    if (pocket == NULL) {
	REPORT_ERROR ("Set pocket flat bottom: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set bottom for pocket %s to flat", pocket->get_its_id ());

    RoseDesign *des = pocket->getRootObject()->design();
    RoseObject *obj = pocket->get_bottom_condition();

    if (Through_pocket_bottom_condition::find (obj)) 
	ARMdelete(Through_pocket_bottom_condition::find (obj));
    else if (Planar_pocket_bottom_condition::find (obj)) 
	ARMdelete(Planar_pocket_bottom_condition::find (obj));
    else if (Radiused_pocket_bottom_condition::find (obj)) 
	ARMdelete(Radiused_pocket_bottom_condition::find (obj));

    Planar_pocket_bottom_condition *flatie = Planar_pocket_bottom_condition::newInstance (des);

    pocket->put_bottom_condition (flatie->getRoot());
    return pocket;
}

Pocket_IF *set_pocket_radiused_bottom (Pocket_IF *pocket, double radius, double x, double y, double z, StixUnit length_unit)
{
    if (pocket == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set pocket flat bottom: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set bottom for pocket %s to radiused (%g %s) at (%g, %g, %g)", pocket->get_its_id (), radius, stix_get_unit_name (length_unit), x, y, z);

    RoseDesign *des = pocket->getRootObject()->design();
    RoseObject *obj = pocket->get_bottom_condition();

    if (Through_pocket_bottom_condition::find (obj)) 
	ARMdelete(Through_pocket_bottom_condition::find (obj));
    else if (Planar_pocket_bottom_condition::find (obj)) 
	ARMdelete(Planar_pocket_bottom_condition::find (obj));
    else if (Radiused_pocket_bottom_condition::find (obj)) 
	ARMdelete(Radiused_pocket_bottom_condition::find (obj));

    Radiused_pocket_bottom_condition *radie = Radiused_pocket_bottom_condition::newInstance (des);
    radie->put_floor_radius (stix_make_length_measure_item (des, radius, length_unit));
    radie->put_floor_radius_center (StixVctr (x, y, z).makePtIn (des));

    pocket->put_bottom_condition (radie->getRoot());
    return pocket;
}

// Open boundary describes the pocket boundary after material has been removed
// Wall boundary describes the pocket boundary before the material has been removed
Open_pocket *new_open_circular_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double radius, double swept_angle, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("New open circular pocket: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open circular pocket for workpiece %s name= %s, pocket depth = %g radius = %g sweep = %g length unit = %s, angle unit = %s ", piece->get_its_id (), nme, pocket_depth, radius, 
			swept_angle, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Open_pocket * pocket = Open_pocket::newInstance (des);
    pocket->put_feature_placement (placement.makeAp3dIn (des));
    pocket->put_its_id (nme);
    pocket->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, pocket_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_course_of_travel (path->getRoot());

    Partial_circular_profile * profile =  Partial_circular_profile::newInstance (des);
    profile->put_radius (stix_make_length_measure_item (des, radius, length_unit));
    profile->put_sweep_angle (stix_make_angle_measure_item (des, swept_angle, angle_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_open_boundary(profile->getRoot());

//    pocket->put_bottom_condition ((Through_pocket_bottom_condition::newInstance(des))->getRoot());

    return pocket;
}

Open_pocket *new_open_linear_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double length, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New open linear pocket: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open circular pocket for workpiece %s name= %s, pocket depth = %g profile length = %g length unit = %s", piece->get_its_id (), nme, pocket_depth, length, 
			stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Open_pocket * pocket = Open_pocket::newInstance (des);
    pocket->put_feature_placement (placement.makeAp3dIn (des));
    pocket->put_its_id (nme);
    pocket->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, pocket_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_course_of_travel (path->getRoot());

    Linear_profile * profile =  Linear_profile::newInstance (des);
    profile->put_profile_length (stix_make_length_measure_item (des, length, length_unit));
    pocket->put_open_boundary(profile->getRoot());

//    pocket->put_bottom_condition ((Through_pocket_bottom_condition::newInstance(des))->getRoot());

    return pocket;
}

Open_pocket *new_open_rounded_u_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double u_width, double u_depth, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New open rounded_u pocket: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open rounded u pocket for workpiece %s name= %s, pocket_depth = %g u_width = %g  u_depth = %g units=%s, ", piece->get_its_id (), nme, pocket_depth, u_width, u_depth, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Open_pocket * pocket = Open_pocket::newInstance (des);
    pocket->put_feature_placement (placement.makeAp3dIn (des));
    pocket->put_its_id (nme);
    pocket->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, pocket_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_course_of_travel (path->getRoot());

    Rounded_u_profile * profile =  Rounded_u_profile::newInstance (des);
    profile->put_width (stix_make_length_measure_item (des, u_width, length_unit));
    profile->put_depth (stix_make_length_measure_item (des, u_depth, length_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_open_boundary(profile->getRoot());

//    pocket->put_bottom_condition ((Through_pocket_bottom_condition::newInstance(des))->getRoot());

    return pocket;
}

Open_pocket *new_open_square_u_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double u_width, double u_first_radius, double u_first_angle, 
			    double u_second_radius, double u_second_angle, double u_depth, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("New open square_u pocket: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open square u pocket for workpiece %s name= %s, pocket_depth = %g u_width = %g  u_fist_radius = %g u_first_angle = %g u_second_radius = %g, u_second_angle = %g, length_unit=%s, angle_unit=%s", 
		piece->get_its_id (), nme, pocket_depth, u_width, u_first_radius, u_first_angle, u_second_radius, u_second_angle, u_depth, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Open_pocket * pocket = Open_pocket::newInstance (des);
    pocket->put_feature_placement (placement.makeAp3dIn (des));
    pocket->put_its_id (nme);
    pocket->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, pocket_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_course_of_travel (path->getRoot());

    Square_u_profile * profile =  Square_u_profile::newInstance (des);
    profile->put_width (stix_make_length_measure_item (des, u_width, length_unit));
    profile->put_depth (stix_make_length_measure_item (des, u_depth, length_unit));
    profile->put_first_radius (stix_make_length_measure_item (des, u_first_radius, length_unit));
    profile->put_second_radius (stix_make_length_measure_item (des, u_second_radius, length_unit));
    profile->put_first_angle (stix_make_angle_measure_item (des, u_first_angle, angle_unit));
    profile->put_second_angle (stix_make_angle_measure_item (des, u_second_angle, angle_unit));

    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_open_boundary(profile->getRoot());

//    pocket->put_bottom_condition ((Through_pocket_bottom_condition::newInstance(des))->getRoot());

    return pocket;
}

/* Tee_profile currently missing from ARM library
Open_pocket *new_open_tee_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double depth, double u_width, double u_first_angle, double u_second_angle, 
	    double cross_bar_width, double cross_bar_depth, double u_radius, double u_depth, double first_offset, double second_offset, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL angle_unit == NULL) {
	REPORT_ERROR ("New open tee pocket: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open square u pocket for workpiece %s name= %s, pocket_depth = %g  u_first_angle = %g u_second_angle = %g cross_bar_width = %g cross_bar_depth = %g u_radius = %g u_depth = %g first_offset = %g second-offset = %g, length_unit=%s, angle_unit=%s", 
		piece->get_its_id (), nme, depth, u_width, u_first_angle, u_second_angle, cross_bar_width, cross_bar_depth, u_radius, u_depth, 
		first_offset, second_offset, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Open_pocket * pocket = Open_pocket::newInstance (des);
    pocket->put_feature_placement (placement.makeAp3dIn (des));
    pocket->put_its_id (nme);
    pocket->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_course_of_travel (path->getRoot());

    Tee_profile * profile =  Tee_profile::newInstance (des);
    profile->put_width (stix_make_length_measure_item (des, u_width, length_unit));
    profile->put_depth (stix_make_length_measure_item (des, u_depth, length_unit));
    profile->put_first_radius (stix_make_length_measure_item (des, u_first_radius, length_unit));
    profile->put_second_radius (stix_make_length_measure_item (des, u_second_radius, length_unit));
    profile->put_first_angle (stix_make_angle_measure_item (des, u_first_angle, angle_unit));
    profile->put_second_angle (stix_make_angle_measure_item (des, u_second_angle, angle_unit));

    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_open_boundary(profile->getRoot());

//    pocket->put_bottom_condition ((Through_pocket_bottom_condition::newInstance(des))->getRoot());

    return pocket;
} */

Open_pocket *new_open_vee_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double profile_radius, double profile_angle, double tilt_angle,
			    double first_side_length, double second_side_length, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("New open vee pocket: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open vee pocket for workpiece %s name= %s, pocket_depth = %g profile_radius = %g profile_angle = %g tilt_angle = %g, first_side_length = %g second_side_length = %g length_unit=%s, angle_unit=%s", 
		piece->get_its_id (), nme, pocket_depth, profile_radius, profile_angle, tilt_angle, first_side_length, second_side_length, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Open_pocket * pocket = Open_pocket::newInstance (des);
    pocket->put_feature_placement (placement.makeAp3dIn (des));
    pocket->put_its_id (nme);
    pocket->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, pocket_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_course_of_travel (path->getRoot());

    Vee_profile * profile =  Vee_profile::newInstance (des);
    profile->put_profile_radius (stix_make_length_measure_item (des, profile_radius, length_unit));
    profile->put_first_side_length (stix_make_length_measure_item (des, first_side_length, length_unit));
    profile->put_second_side_length (stix_make_length_measure_item (des, second_side_length, length_unit));
    profile->put_profile_angle (stix_make_angle_measure_item (des, profile_angle, angle_unit));
    profile->put_tilt_angle (stix_make_angle_measure_item (des, tilt_angle, angle_unit));

    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_open_boundary(profile->getRoot());

//    pocket->put_bottom_condition ((Through_pocket_bottom_condition::newInstance(des))->getRoot());

    return pocket;
}

Open_pocket *new_open_path_pocket (Workpiece *piece, char * nme,  StixMtrx placement, stp_bounded_curve *curve, double pocket_depth, StixUnit length_unit)
{
    if (piece == NULL || curve == NULL || length_unit == NULL) {
	REPORT_ERROR ("New open path pocket: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open path pocket for design %s name= %s, depth = %g %s", piece->get_its_id (), nme, pocket_depth, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject ()->design ();

    // Could check the profile
    // If a polyline then more than two points

    Open_pocket * pocket = Open_pocket::newInstance (des);
    pocket->put_feature_placement (placement.makeAp3dIn (des));
    pocket->put_its_id (nme);
    pocket->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, pocket_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_course_of_travel (path->getRoot());

    General_profile * profile = General_profile::newInstance (des);
    profile->put_its_profile (curve); 
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_open_boundary(profile->getRoot());

//    pocket->put_bottom_condition ((Through_pocket_bottom_condition::newInstance(des))->getRoot());

    return pocket;
}

// Open boundary describes the pocket boundary after material has been removed
// Wall boundary describes the pocket boundary before the material has been removed
Open_pocket *set_open_circular_wall (Open_pocket *pocket, double radius, double swept_angle, StixUnit length_unit, StixUnit angle_unit)
{
    if (pocket == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("Set open circular wall: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set open circular wall for pocket %s radius = %g sweep = %g length unit = %s, angle unit = %s ", pocket->get_its_id (), radius, swept_angle, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = pocket->getRootObject()->design();    
    
    Partial_circular_profile * profile =  Partial_circular_profile::newInstance (des);
    profile->put_radius (stix_make_length_measure_item (des, radius, length_unit));
    profile->put_sweep_angle (stix_make_angle_measure_item (des, swept_angle, angle_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_wall_boundary(profile->getRoot());

    return pocket;
}

Open_pocket *set_open_linear_wall (Open_pocket *pocket, double length, StixUnit length_unit)
{
    if (pocket == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set open linear wall: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set open linear wall for pocket %s profile length = %g length unit = %s", pocket->get_its_id (), length, stix_get_unit_name (length_unit));

    RoseDesign *des = pocket->getRootObject()->design();    
    
    Linear_profile * profile =  Linear_profile::newInstance (des);
    profile->put_profile_length (stix_make_length_measure_item (des, length, length_unit));
    pocket->put_wall_boundary(profile->getRoot());

    return pocket;
}

Open_pocket *set_open_rounded_u_wall (Open_pocket *pocket, char * nme, double u_width, double u_depth, StixUnit length_unit)
{
    if (pocket == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set open rounded_u wall: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set open rounded u wall for pocket %s u_width = %g  u_depth = %g units = %s, ", pocket->get_its_id (), u_width, u_depth, stix_get_unit_name (length_unit));

    RoseDesign *des = pocket->getRootObject()->design();    

    Rounded_u_profile * profile =  Rounded_u_profile::newInstance (des);
    profile->put_width (stix_make_length_measure_item (des, u_width, length_unit));
    profile->put_depth (stix_make_length_measure_item (des, u_depth, length_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_wall_boundary(profile->getRoot());

    return pocket;
}

Open_pocket *set_open_square_u_wall (Open_pocket *pocket, double u_width, double u_first_radius, double u_first_angle, double u_second_radius, double u_second_angle, double u_depth, StixUnit length_unit, StixUnit angle_unit)
{
    if (pocket == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("Set open square_u wall: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set open square u wall for pocket %s u_width = %g  u_first_radius = %g u_first_angle = %g u_second_radius = %g, u_second_angle = %g, length_unit=%s, angle_unit=%s", 
		pocket->get_its_id (), u_width, u_first_radius, u_first_angle, u_second_radius, u_second_angle, u_depth, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = pocket->getRootObject()->design();    
    
    Square_u_profile * profile =  Square_u_profile::newInstance (des);
    profile->put_width (stix_make_length_measure_item (des, u_width, length_unit));
    profile->put_depth (stix_make_length_measure_item (des, u_depth, length_unit));
    profile->put_first_radius (stix_make_length_measure_item (des, u_first_radius, length_unit));
    profile->put_second_radius (stix_make_length_measure_item (des, u_second_radius, length_unit));
    profile->put_first_angle (stix_make_angle_measure_item (des, u_first_angle, angle_unit));
    profile->put_second_angle (stix_make_angle_measure_item (des, u_second_angle, angle_unit));

    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_wall_boundary(profile->getRoot());

    return pocket;
}

/* Tee_profile currently missing from ARM library
Open_pocket *set_open_tee_wall (Open_pocket *pocket, double u_width, double u_first_angle, double u_second_angle, 
	    double cross_bar_width, double cross_bar_depth, double u_radius, double u_depth, double first_offset, double second_offset, StixUnit length_unit, StixUnit angle_unit)
{
    if (pocket == NULL || length_unit == NULL angle_unit == NULL) {
	REPORT_ERROR ("Set open tee wall: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Seet open square u wall for pocket %s u_first_angle = %g u_second_angle = %g cross_bar_width = %g cross_bar_depth = %g u_radius = %g u_depth = %g first_offset = %g second-offset = %g, length_unit=%s, angle_unit=%s", 
		pocket->get_its_id (), u_width, u_first_angle, u_second_angle, cross_bar_width, cross_bar_depth, u_radius, u_depth, 
		first_offset, second_offset, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Tee_profile * profile =  Tee_profile::newInstance (des);
    profile->put_width (stix_make_length_measure_item (des, u_width, length_unit));
    profile->put_depth (stix_make_length_measure_item (des, u_depth, length_unit));
    profile->put_first_radius (stix_make_length_measure_item (des, u_first_radius, length_unit));
    profile->put_second_radius (stix_make_length_measure_item (des, u_second_radius, length_unit));
    profile->put_first_angle (stix_make_angle_measure_item (des, u_first_angle, angle_unit));
    profile->put_second_angle (stix_make_angle_measure_item (des, u_second_angle, angle_unit));

    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_wall_boundary(profile->getRoot());

    return pocket;
} */

Open_pocket *set_open_vee_wall (Open_pocket *pocket, double profile_radius, double profile_angle, double tilt_angle,
			    double first_side_length, double second_side_length, StixUnit length_unit, StixUnit angle_unit)
{
    if (pocket == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("Set open vee wall: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set open vee wall for pocket %s profile_radius = %g profile_angle = %g tilt_angle = %g, first_side_length = %g second_side_length = %g length_unit=%s, angle_unit=%s", 
		pocket->get_its_id (), profile_radius, profile_angle, tilt_angle, first_side_length, second_side_length, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = pocket->getRootObject()->design();    
    
    Vee_profile * profile =  Vee_profile::newInstance (des);
    profile->put_profile_radius (stix_make_length_measure_item (des, profile_radius, length_unit));
    profile->put_first_side_length (stix_make_length_measure_item (des, first_side_length, length_unit));
    profile->put_second_side_length (stix_make_length_measure_item (des, second_side_length, length_unit));
    profile->put_profile_angle (stix_make_angle_measure_item (des, profile_angle, angle_unit));
    profile->put_tilt_angle (stix_make_angle_measure_item (des, tilt_angle, angle_unit));

    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_wall_boundary(profile->getRoot());

    return pocket;
}

Open_pocket *set_open_path_wall (Open_pocket *pocket, stp_bounded_curve *curve)
{
    if (pocket == NULL || curve == NULL) {
	REPORT_ERROR ("Set open path wall: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set open path wall for pocket %s", pocket->get_its_id ());

    RoseDesign *des = pocket->getRootObject ()->design ();

    // Could check the profile
    // If a polyline then more than two points

    General_profile * profile = General_profile::newInstance (des);
    profile->put_its_profile (curve); 
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    pocket->put_wall_boundary(profile->getRoot());

    return pocket;
}

Rectangular_boss *new_rectangular_boss (Workpiece *piece, char * nme, StixMtrx placement, double height, double length, double width, StixUnit length_unit) 
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New rectangular boss: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New rectangular boss for workpiece %s name = %s, height = %g length = %g width = %g unit = %s", piece->get_its_id (), nme, height, length, width, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject ()->design ();

    Rectangular_boss * boss = Rectangular_boss::newInstance (des);
    boss->put_feature_placement (placement.makeAp3dIn (des));
    boss->put_its_id (nme);
    boss->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, height, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    boss->put_boss_height (path->getRoot());

    Rectangular_closed_profile * profile = Rectangular_closed_profile::newInstance (des);
    profile->put_profile_length (stix_make_length_measure_item (des, length, length_unit));
    profile->put_profile_width (stix_make_length_measure_item (des, width, length_unit));;
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    boss->put_its_boundary(profile->getRoot());

    return boss;
}

Circular_boss *new_circular_boss (Workpiece *piece, char * nme, StixMtrx placement, double height, double diameter, StixUnit length_unit) 
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New circular boss: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New circular boss for workpiece %s name = %s, height = %g diameter = %g unit = %s", piece->get_its_id (), nme, height, diameter, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject ()->design ();

    Circular_boss * boss = Circular_boss::newInstance (des);
    boss->put_feature_placement (placement.makeAp3dIn (des));
    boss->put_its_id (nme);
    boss->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, height, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    boss->put_boss_height (path->getRoot());

    Circular_closed_profile * profile = Circular_closed_profile::newInstance (des);
    profile->put_diameter (stix_make_length_measure_item (des, diameter, length_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    boss->put_its_boundary(profile->getRoot());

    return boss;
}

Complex_boss *new_path_boss (Workpiece *piece, char * nme, StixMtrx placement, double height, stp_bounded_curve *curve, StixUnit length_unit) 
{
    if (piece == NULL || curve == NULL) {
	REPORT_ERROR ("New path boss: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New path boss for workpiece %s name = %s, height = %g unit = %s", piece->get_its_id (), nme, height, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject ()->design ();

    Complex_boss * boss = Complex_boss::newInstance (des);
    boss->put_feature_placement (placement.makeAp3dIn (des));
    boss->put_its_id (nme);
    boss->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, height, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    boss->put_boss_height (path->getRoot());

    General_closed_profile * profile = General_closed_profile::newInstance (des);
    profile->put_closed_profile_shape (curve); 
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    boss->put_its_boundary(profile->getRoot());

    return boss;
}

Boss_IF *set_boss_slope (Boss_IF *boss, double slope, StixUnit angle_unit) 
{
    if (boss == NULL || angle_unit == NULL) {
	REPORT_ERROR ("Set boss slope: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set slope for boss %s to %g with units = %s", boss->get_its_id (), slope, stix_get_unit_name (angle_unit));

    RoseDesign *des = boss->getRootObject()->design(); 
    Angle_taper * taper = Angle_taper::newInstance(des);
    taper->put_angle (stix_make_angle_measure_item (des, slope, angle_unit));
    boss->put_slope (taper->getRoot());
    return boss;
}

Pocket_IF *add_boss_to_pocket (Pocket_IF *pocket, Boss_IF *boss) 
{
    if (boss == NULL || pocket == NULL) {
	REPORT_ERROR ("Add boss to pocket: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Adding boss %s to pocket %s", pocket->get_its_id (), boss->get_its_id ());

    pocket->add_its_boss (ROSE_CAST (stp_boss_and_instanced_feature, boss->getRootObject ()));
    return pocket;
}

Step *add_boss_to_step (Step *step, Boss_IF *boss) 
{
    if (boss == NULL || step == NULL) {
	REPORT_ERROR ("Add boss to step: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Adding boss %s to step %s", step->get_its_id (), boss->get_its_id ());

    step->add_its_boss (ROSE_CAST (stp_boss_and_instanced_feature, boss->getRootObject ()));
    return step;
}

Planar_face *add_boss_to_face (Planar_face *face, Boss_IF *boss) 
{
    if (boss == NULL || face == NULL) {
	REPORT_ERROR ("Add boss to planar face: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Adding boss %s to planar face %s", face->get_its_id (), boss->get_its_id ());

    face->add_its_boss (ROSE_CAST (stp_boss_and_instanced_feature, boss->getRootObject ()));
    return face;
}

Planar_face *new_planar_face (Workpiece *piece, char * nme, StixMtrx placement, double depth, double length, double width, StixUnit length_unit) 
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New planar face: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New planar face for workpiece %s name = %s, depth = %g length = %g width = %g unit = %s", piece->get_its_id (), nme, depth, length, width, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject ()->design ();

    Planar_face *face = Planar_face::newInstance (des);
    face->put_feature_placement (placement.makeAp3dIn (des));
    face->put_its_id (nme);
    face->put_its_workpiece (piece->getRoot());
    face->put_removal_depth (stix_make_length_measure_item (des, depth, length_unit));
    face->put_removal_direction (placement.z().makeDirIn (des));

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, width, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    face->put_course_of_travel (path->getRoot());

    Linear_profile * profile = Linear_profile::newInstance (des);
    profile->put_profile_length (stix_make_length_measure_item (des, length, length_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    face->put_removal_boundary(profile->getRoot());
    return face;   
}

Planar_face *set_face_circular_boundary (Planar_face *face, StixMtrx placement, double diameter, StixUnit length_unit)
{
    if (face == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set circular boundary for planar face: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set circular boundary for planar face name = %s, diameter = %g unit = %s", face->get_its_id (), diameter, stix_get_unit_name (length_unit));

    RoseDesign *des = face->getRootObject ()->design ();

    Circular_closed_profile * profile = Circular_closed_profile::newInstance (des);
    profile->put_diameter (stix_make_length_measure_item (des, diameter, length_unit));
    profile->put_placement (placement.makeAp3dIn (des));
    face->put_face_boundary(profile->getRoot());

    return face;   
}

Planar_face *set_face_rectangular_boundary (Planar_face *face, StixMtrx placement, double length, double width, StixUnit length_unit)
{
    if (face == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set rectangular boundary for planar face: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set rectangular boundary for planar face name = %s, length = %g width = %g unit = %s", face->get_its_id (), length, width, stix_get_unit_name (length_unit));

    RoseDesign *des = face->getRootObject ()->design ();

    Rectangular_closed_profile * profile = Rectangular_closed_profile::newInstance (des);
    profile->put_profile_length (stix_make_length_measure_item (des, length, length_unit));
    profile->put_profile_width (stix_make_length_measure_item (des, width, length_unit));
    profile->put_placement (placement.makeAp3dIn (des));
    face->put_face_boundary(profile->getRoot());

    return face;   
}

Planar_face *set_face_path_boundary (Planar_face *face, StixMtrx placement, stp_bounded_curve *curve)
{
    if (face == NULL) {
	REPORT_ERROR ("Set path boundary for planar face: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set path boundary for planar face name = %s", face->get_its_id ());

    RoseDesign *des = face->getRootObject ()->design ();

    General_closed_profile * profile = General_closed_profile::newInstance (des);
    profile->put_closed_profile_shape (curve); 
    profile->put_placement (placement.makeAp3dIn (des));
    face->put_face_boundary(profile->getRoot());

    return face;   
}

// Outside profiles do not have a floor condition
// Shape profiles have a floor condition
General_outside_profile *new_closed_rectangular_outside_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, double length, double width, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New rectangular outside profile: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New rectangular outside profile for workpiece %s name= %s, feature_depth = %g length, width = %g,%g  units=%s", piece->get_its_id (), nme, feature_depth, length, width, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    General_outside_profile * feature = General_outside_profile::newInstance (des);
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, feature_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_swept_shape (path->getRoot());

    Rectangular_closed_profile * profile = Rectangular_closed_profile::newInstance (des);
    profile->put_profile_length (stix_make_length_measure_item (des, length, length_unit));
    profile->put_profile_width (stix_make_length_measure_item (des, width, length_unit));;
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_feature_boundary(profile->getRoot());

    return feature;
}

General_outside_profile *new_closed_circular_outside_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, double diameter, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New closed circular outside profile: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New closed circular outside profile for workpiece %s name= %s, feature_depth = %g diameter = %g  units=%s", piece->get_its_id (), nme, feature_depth, diameter, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    General_outside_profile * feature = General_outside_profile::newInstance (des);
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, feature_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_swept_shape (path->getRoot());

    Circular_closed_profile * profile = Circular_closed_profile::newInstance (des);
    profile->put_diameter (stix_make_length_measure_item (des, diameter, length_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_feature_boundary(profile->getRoot());

    return feature;
}

General_outside_profile *new_partial_circular_outside_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, double radius, double sweep_angle, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("New partial circular outside profile: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New partial circular outside profile for workpiece %s name= %s, feature_depth = %g radius = %g  %s sweep angle = %g %s", piece->get_its_id (), nme, feature_depth, radius, stix_get_unit_name (length_unit),
			sweep_angle, stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    General_outside_profile * feature = General_outside_profile::newInstance (des);
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, feature_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_swept_shape (path->getRoot());

    Partial_circular_profile * profile = Partial_circular_profile::newInstance (des);
    profile->put_radius (stix_make_length_measure_item (des, radius, length_unit));
    profile->put_sweep_angle (stix_make_angle_measure_item (des, sweep_angle, angle_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_feature_boundary(profile->getRoot());

    return feature;
}

General_outside_profile *new_linear_outside_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, double length, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New linear outside profile: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New linear outside profile for workpiece %s name= %s, feature_depth = %g length = %g  units=%s", piece->get_its_id (), nme, feature_depth, length, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    General_outside_profile * feature = General_outside_profile::newInstance (des);
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, feature_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_swept_shape (path->getRoot());

    Linear_profile * profile = Linear_profile::newInstance (des);
    profile->put_profile_length (stix_make_length_measure_item (des, length, length_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_feature_boundary(profile->getRoot());

    return feature;
}

General_outside_profile *new_closed_path_outside_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, stp_bounded_curve *curve, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL || curve == NULL) {
	REPORT_ERROR ("New closed path outside profile: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New closed path outside profile for workpiece %s name= %s, feature_depth = %g units=%s", piece->get_its_id (), nme, feature_depth, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    General_outside_profile * feature = General_outside_profile::newInstance (des);
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, feature_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_swept_shape (path->getRoot());

    General_closed_profile * profile = General_closed_profile::newInstance (des);
    profile->put_closed_profile_shape (curve); 
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_feature_boundary(profile->getRoot());

    return feature;
}

General_outside_profile *new_open_path_outside_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, stp_bounded_curve *curve, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL || curve == NULL) {
	REPORT_ERROR ("New open path outside profile: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open path outside profile for workpiece %s name= %s, feature_depth = %g units=%s", piece->get_its_id (), nme, feature_depth, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    General_outside_profile * feature = General_outside_profile::newInstance (des);
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, feature_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_swept_shape (path->getRoot());

    General_profile * profile = General_profile::newInstance (des);
    profile->put_its_profile (curve); 
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_feature_boundary(profile->getRoot());

    return feature;
}

Rectangular_closed_shape_profile *new_rectangular_closed_shape_profile (Workpiece *piece, char * nme,  StixMtrx placement, StixVctr removal_direction, double feature_depth, double length, double width, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New rectangular closed shape profile: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New rectangular closed shape profile for workpiece %s name= %s, feature_depth = %g length, width = %g,%g  units=%s", piece->get_its_id (), nme, feature_depth, length, width, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Rectangular_closed_shape_profile * feature = Rectangular_closed_shape_profile::newInstance (des);
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, feature_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_swept_shape (path->getRoot());

    Rectangular_closed_profile * profile = Rectangular_closed_profile::newInstance (des);
    profile->put_profile_length (stix_make_length_measure_item (des, length, length_unit));
    profile->put_profile_width (stix_make_length_measure_item (des, width, length_unit));;
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_closed_boundary(profile->getRoot());
    feature->put_removal_direction (removal_direction.makeDirIn (des));

    return feature;
}

Circular_closed_shape_profile *new_circular_closed_shape_profile (Workpiece *piece, char * nme,  StixMtrx placement, StixVctr removal_direction, double feature_depth, double diameter, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New circular closed shape profile: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New circular closed shape profile for workpiece %s name= %s, feature_depth = %g diameter = %g  units=%s", piece->get_its_id (), nme, feature_depth, diameter, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Circular_closed_shape_profile * feature = Circular_closed_shape_profile::newInstance (des);
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, feature_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_swept_shape (path->getRoot());

    Circular_closed_profile * profile = Circular_closed_profile::newInstance (des);
    profile->put_diameter (stix_make_length_measure_item (des, diameter, length_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_closed_boundary(profile->getRoot());
    feature->put_removal_direction (removal_direction.makeDirIn (des));

    return feature;
}

Partial_circular_shape_profile *new_partial_circular_shape_profile (Workpiece *piece, char * nme,  StixMtrx placement, StixVctr removal_direction, double feature_depth, double radius, double sweep_angle, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("New partial circular shape profile: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New partial circular shape profile for workpiece %s name= %s, feature_depth = %g radius = %g  %s sweep angle = %g %s", piece->get_its_id (), nme, feature_depth, radius, stix_get_unit_name (length_unit),
			sweep_angle, stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Partial_circular_shape_profile * feature = Partial_circular_shape_profile::newInstance (des);
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, feature_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_swept_shape (path->getRoot());

    Partial_circular_profile * profile = Partial_circular_profile::newInstance (des);
    profile->put_radius (stix_make_length_measure_item (des, radius, length_unit));
    profile->put_sweep_angle (stix_make_angle_measure_item (des, sweep_angle, angle_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_open_boundary(profile->getRoot());
    feature->put_removal_direction (removal_direction.makeDirIn (des));

    return feature;
}

Rectangular_open_shape_profile *new_rectangular_open_shape_profile (Workpiece *piece, char * nme,  StixMtrx placement, StixVctr removal_direction, double feature_depth, double u_width, double u_first_radius, double u_first_angle, 
			    double u_second_radius, double u_second_angle, double u_depth, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New linear outside profile: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New rectangular open shape (square u) shape profile for workpiece %s name= %s, pocket_depth = %g u_width = %g  u_fist_radius = %g u_first_angle = %g u_second_radius = %g, u_second_angle = %g, length_unit=%s, angle_unit=%s", 
		piece->get_its_id (), nme, feature_depth, u_width, u_first_radius, u_first_angle, u_second_radius, u_second_angle, u_depth, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Rectangular_open_shape_profile * feature = Rectangular_open_shape_profile::newInstance (des);
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, feature_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_swept_shape (path->getRoot());

    Square_u_profile * profile =  Square_u_profile::newInstance (des);
    profile->put_width (stix_make_length_measure_item (des, u_width, length_unit));
    profile->put_depth (stix_make_length_measure_item (des, u_depth, length_unit));
    profile->put_first_radius (stix_make_length_measure_item (des, u_first_radius, length_unit));
    profile->put_second_radius (stix_make_length_measure_item (des, u_second_radius, length_unit));
    profile->put_first_angle (stix_make_angle_measure_item (des, u_first_angle, angle_unit));
    profile->put_second_angle (stix_make_angle_measure_item (des, u_second_angle, angle_unit));
    feature->put_open_boundary(profile->getRoot());
    feature->put_removal_direction (removal_direction.makeDirIn (des));

    return feature;
}

General_shape_profile *new_closed_path_shape_profile (Workpiece *piece, char * nme,  StixMtrx placement, StixVctr removal_direction, double feature_depth, stp_bounded_curve *curve, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL || curve == NULL) {
	REPORT_ERROR ("New closed path shape profile: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New closed path shape profile for workpiece %s name= %s, feature_depth = %g units=%s", piece->get_its_id (), nme, feature_depth, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    General_shape_profile * feature = General_shape_profile::newInstance (des);
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, feature_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_swept_shape (path->getRoot());

    General_closed_profile * profile = General_closed_profile::newInstance (des);
    profile->put_closed_profile_shape (curve); 
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_boundary(profile->getRoot());
    feature->put_removal_direction (removal_direction.makeDirIn (des));

    return feature;
}

General_shape_profile *new_open_path_shape_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, stp_bounded_curve *curve, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL || curve == NULL) {
	REPORT_ERROR ("New open path shape profile: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open path shape profile for workpiece %s name= %s, feature_depth = %g units=%s", piece->get_its_id (), nme, feature_depth, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    General_shape_profile * feature = General_shape_profile::newInstance (des);
    feature->put_feature_placement (placement.makeAp3dIn (des));
    feature->put_its_id (nme);
    feature->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, feature_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_swept_shape (path->getRoot());

    General_profile * profile = General_profile::newInstance (des);
    profile->put_its_profile (curve); 
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    feature->put_profile_boundary(profile->getRoot());

    return feature;
}

Planar_profile_floor *new_planar_profile_floor (Workpiece *piece, stp_plane *plane, const char *start_or_end, double floor_radius, StixUnit length_unit)
{
    if (piece == NULL || plane == NULL || start_or_end == NULL || length_unit == NULL) {
	REPORT_ERROR ("New planar profile floor: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New planar profile floor for workpiece %s, radius = %g (%s) start_or_end = %s", piece->get_its_id (), floor_radius, stix_get_unit_name (length_unit), start_or_end);

    RoseDesign *des = piece->getRootObject()->design();    
    Planar_profile_floor *floor = Planar_profile_floor::newInstance (des);
    floor->put_start_or_end (start_or_end);			    // EN_SHAPE_PROFILE_END, EN_SHAPE_PROFILE_START
    if (floor_radius > 0)
	floor->put_floor_radius (stix_make_length_measure_item (des, floor_radius, length_unit));
    floor->put_floor (plane);
    return floor;
}

General_profile_floor *new_general_profile_floor (Workpiece *piece, stp_face *face, const char *start_or_end, double floor_radius, StixUnit length_unit)
{
    if (piece == NULL || face == NULL || start_or_end == NULL || length_unit == NULL) {
	REPORT_ERROR ("New general profile floor: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New general profile floor for workpiece %s, radius = %g (%s) start_or_end = %s", piece->get_its_id (), floor_radius, stix_get_unit_name (length_unit), start_or_end);

    RoseDesign *des = piece->getRootObject()->design();    
    General_profile_floor *floor = General_profile_floor::newInstance (des);
    floor->put_start_or_end (start_or_end);			    // EN_SHAPE_PROFILE_END, EN_SHAPE_PROFILE_START
    if (floor_radius > 0)
	floor->put_floor_radius (stix_make_length_measure_item (des, floor_radius, length_unit));
    floor->put_floor (face);
    return floor;
}

Through_profile_floor *new_through_profile_floor (Workpiece *piece)
{
    if (piece == NULL) {
	REPORT_ERROR ("New through profile floor: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New through profile floor for workpiece %s", piece->get_its_id ());

    RoseDesign *des = piece->getRootObject()->design();    
    Through_profile_floor *floor = Through_profile_floor::newInstance (des);

    return floor;
}

Shape_profile_IF *set_shape_profile_floor (Shape_profile_IF *feature, Profile_floor_IF *floor)
{
    if (feature == NULL || floor == NULL) {
	REPORT_ERROR ("Set shape profile floor: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set shape profile floor for feature %s, to %s", feature->get_its_id (), floor->getModuleName ());

    feature->put_floor_condition (ROSE_CAST (stp_profile_floor, floor->getRootObject()));
    return feature;
}

Slot *new_open_circular_slot (Workpiece *piece, char * nme,  StixMtrx placement, double slot_depth, double radius, double swept_angle, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("New open circular slot: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open circular slot for workpiece %s name= %s, slot depth = %g radius = %g sweep = %g length unit = %s, angle unit = %s ", piece->get_its_id (), nme, slot_depth, radius, 
			swept_angle, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Slot *slot = Slot::newInstance (des);
    slot->put_feature_placement (placement.makeAp3dIn (des));
    slot->put_its_id (nme);
    slot->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, slot_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_course_of_travel (path->getRoot());

    Partial_circular_profile * profile =  Partial_circular_profile::newInstance (des);
    profile->put_radius (stix_make_length_measure_item (des, radius, length_unit));
    profile->put_sweep_angle (stix_make_angle_measure_item (des, swept_angle, angle_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_swept_shape (profile->getRoot());

    return slot;
}

Slot *new_open_linear_slot (Workpiece *piece, char * nme,  StixMtrx placement, double slot_depth, double length, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New open linear slot: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open linear slot for workpiece %s name= %s, slot depth = %g profile length = %g length unit = %s", piece->get_its_id (), nme, slot_depth, length, 
			stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Slot * slot = Slot::newInstance (des);
    slot->put_feature_placement (placement.makeAp3dIn (des));
    slot->put_its_id (nme);
    slot->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, slot_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_course_of_travel (path->getRoot());

    Linear_profile * profile =  Linear_profile::newInstance (des);
    profile->put_profile_length (stix_make_length_measure_item (des, length, length_unit));
    slot->put_swept_shape (profile->getRoot());

    return slot;
}

Slot *new_open_rounded_u_slot (Workpiece *piece, char * nme,  StixMtrx placement, double slot_depth, double u_width, double u_depth, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New open rounded_u slot: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open rounded u slot for workpiece %s name= %s, slot_depth = %g u_width = %g  u_depth = %g units=%s, ", piece->get_its_id (), nme, slot_depth, u_width, u_depth, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Slot * slot = Slot::newInstance (des);
    slot->put_feature_placement (placement.makeAp3dIn (des));
    slot->put_its_id (nme);
    slot->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, slot_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_course_of_travel (path->getRoot());

    Rounded_u_profile * profile =  Rounded_u_profile::newInstance (des);
    profile->put_width (stix_make_length_measure_item (des, u_width, length_unit));
    profile->put_depth (stix_make_length_measure_item (des, u_depth, length_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_swept_shape(profile->getRoot());

    return slot;
}

Slot *new_open_square_u_slot (Workpiece *piece, char * nme,  StixMtrx placement, double slot_depth, double u_width, double u_first_radius, double u_first_angle, 
			    double u_second_radius, double u_second_angle, double u_depth, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("New open square_u slot: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open square u slot for workpiece %s name= %s, pocket_depth = %g u_width = %g  u_fist_radius = %g u_first_angle = %g u_second_radius = %g, u_second_angle = %g, length_unit=%s, angle_unit=%s", 
		piece->get_its_id (), nme, slot_depth, u_width, u_first_radius, u_first_angle, u_second_radius, u_second_angle, u_depth, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Slot * slot = Slot::newInstance (des);
    slot->put_feature_placement (placement.makeAp3dIn (des));
    slot->put_its_id (nme);
    slot->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, slot_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_course_of_travel (path->getRoot());

    Square_u_profile * profile =  Square_u_profile::newInstance (des);
    profile->put_width (stix_make_length_measure_item (des, u_width, length_unit));
    profile->put_depth (stix_make_length_measure_item (des, u_depth, length_unit));
    profile->put_first_radius (stix_make_length_measure_item (des, u_first_radius, length_unit));
    profile->put_second_radius (stix_make_length_measure_item (des, u_second_radius, length_unit));
    profile->put_first_angle (stix_make_angle_measure_item (des, u_first_angle, angle_unit));
    profile->put_second_angle (stix_make_angle_measure_item (des, u_second_angle, angle_unit));

    profile->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_swept_shape (profile->getRoot());

    return slot;
}

/* Tee_profile currently missing from ARM library
Slot *new_open_tee_slot (Workpiece *piece, char * nme,  StixMtrx placement, double slot_depth, double u_width, double u_first_angle, double u_second_angle, 
	    double cross_bar_width, double cross_bar_depth, double u_radius, double u_depth, double first_offset, double second_offset, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL angle_unit == NULL) {
	REPORT_ERROR ("New open tee pocket: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open square u pocket for workpiece %s name= %s, slot_depth = %g  u_first_angle = %g u_second_angle = %g cross_bar_width = %g cross_bar_depth = %g u_radius = %g u_depth = %g first_offset = %g second-offset = %g, length_unit=%s, angle_unit=%s", 
		piece->get_its_id (), nme, slot_depth, u_width, u_first_angle, u_second_angle, cross_bar_width, cross_bar_depth, u_radius, u_depth, 
		first_offset, second_offset, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Slot * pocket = Slot::newInstance (des);
    slot->put_feature_placement (placement.makeAp3dIn (des));
    slot->put_its_id (nme);
    slot->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, slot_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_course_of_travel (path->getRoot());

    Tee_profile * profile =  Tee_profile::newInstance (des);
    profile->put_width (stix_make_length_measure_item (des, u_width, length_unit));
    profile->put_depth (stix_make_length_measure_item (des, u_depth, length_unit));
    profile->put_first_radius (stix_make_length_measure_item (des, u_first_radius, length_unit));
    profile->put_second_radius (stix_make_length_measure_item (des, u_second_radius, length_unit));
    profile->put_first_angle (stix_make_angle_measure_item (des, u_first_angle, angle_unit));
    profile->put_second_angle (stix_make_angle_measure_item (des, u_second_angle, angle_unit));

    profile->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_swept_shape (profile->getRoot());

    return slot;
} */

Slot *new_open_vee_slot (Workpiece *piece, char * nme,  StixMtrx placement, double slot_depth, double profile_radius, double profile_angle, double tilt_angle,
			    double first_side_length, double second_side_length, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("New open vee slot: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open vee slot for workpiece %s name= %s, pocket_depth = %g profile_radius = %g profile_angle = %g tilt_angle = %g, first_side_length = %g second_side_length = %g length_unit=%s, angle_unit=%s", 
		piece->get_its_id (), nme, slot_depth, profile_radius, profile_angle, tilt_angle, first_side_length, second_side_length, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Slot * slot = Slot::newInstance (des);
    slot->put_feature_placement (placement.makeAp3dIn (des));
    slot->put_its_id (nme);
    slot->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, slot_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_course_of_travel (path->getRoot());

    Vee_profile * profile =  Vee_profile::newInstance (des);
    profile->put_profile_radius (stix_make_length_measure_item (des, profile_radius, length_unit));
    profile->put_first_side_length (stix_make_length_measure_item (des, first_side_length, length_unit));
    profile->put_second_side_length (stix_make_length_measure_item (des, second_side_length, length_unit));
    profile->put_profile_angle (stix_make_angle_measure_item (des, profile_angle, angle_unit));
    profile->put_tilt_angle (stix_make_angle_measure_item (des, tilt_angle, angle_unit));

    profile->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_swept_shape (profile->getRoot());

    return slot;
}

Slot *new_open_path_slot (Workpiece *piece, char * nme,  StixMtrx placement, stp_bounded_curve *curve, double slot_depth, StixUnit length_unit)
{
    if (piece == NULL || curve == NULL || length_unit == NULL) {
	REPORT_ERROR ("New open path slot: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New open path slot for design %s name= %s, depth = %g %s", piece->get_its_id (), nme, slot_depth, stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject ()->design ();

    // Could check the profile
    // If a polyline then more than two points

    Slot * slot = Slot::newInstance (des);
    slot->put_feature_placement (placement.makeAp3dIn (des));
    slot->put_its_id (nme);
    slot->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, slot_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_course_of_travel (path->getRoot());

    General_profile * profile = General_profile::newInstance (des);
    profile->put_its_profile (curve); 
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    slot->put_swept_shape (profile->getRoot());

    return slot;
}

Slot *set_open_slot_end_type_at_start (Slot *slot) 
{
    if (slot == NULL) {
	REPORT_ERROR ("Set open slot end type at start: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set slot type at start for slot %s to open end type", slot->get_its_id ());

    RoseDesign *des = slot->getRootObject()->design(); 
    Open_slot_end_type *type = Open_slot_end_type::newInstance(des);
    slot->put_start_condition (type->getRoot());
    return slot;
}

Slot *set_open_slot_end_type_at_end (Slot *slot) 
{
    if (slot == NULL) {
	REPORT_ERROR ("Set open slot end type at end: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set slot type at end for slot %s to open end type", slot->get_its_id ());

    RoseDesign *des = slot->getRootObject()->design(); 
    Open_slot_end_type *type = Open_slot_end_type::newInstance(des);
    slot->put_end_condition (type->getRoot());
    return slot;
}

Slot *set_radiused_slot_end_type_at_start (Slot *slot) 
{
    if (slot == NULL) {
	REPORT_ERROR ("Set radiused slot end type at start: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set slot type at start for slot %s to radiused end type", slot->get_its_id ());

    RoseDesign *des = slot->getRootObject()->design(); 
    Radiused_slot_end_type *type = Radiused_slot_end_type::newInstance(des);
    slot->put_start_condition (type->getRoot());
    return slot;
}

Slot *set_radiused_slot_end_type_at_end (Slot *slot) 
{
    if (slot == NULL) {
	REPORT_ERROR ("Set radiused slot end type at end: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set slot type at end for slot %s to radiused end type", slot->get_its_id ());

    RoseDesign *des = slot->getRootObject()->design(); 
    Radiused_slot_end_type *type = Radiused_slot_end_type::newInstance(des);
    slot->put_end_condition (type->getRoot());
    return slot;
}

Slot *set_loop_slot_end_type_at_start (Slot *slot) 
{
    if (slot == NULL) {
	REPORT_ERROR ("Set loop slot end type at start: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set slot type at start for slot %s to loop end type", slot->get_its_id ());

    RoseDesign *des = slot->getRootObject()->design(); 
    Loop_slot_end_type *type = Loop_slot_end_type::newInstance(des);
    slot->put_start_condition (type->getRoot());
    return slot;
}

Slot *set_loop_slot_end_type_at_end (Slot *slot) 
{
    if (slot == NULL) {
	REPORT_ERROR ("Set loop slot end type at end: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set slot type at end for slot %s to loop end type", slot->get_its_id ());

    RoseDesign *des = slot->getRootObject()->design(); 
    Loop_slot_end_type *type = Loop_slot_end_type::newInstance(des);
    slot->put_end_condition (type->getRoot());
    return slot;
}

Slot *set_woodruff_slot_end_type_at_start (Slot *slot, double radius, StixUnit length_unit) 
{
    if (slot == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set woodruff slot end type at start: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set slot type at start for slot %s to woodruff end type with radius %g (%s)", slot->get_its_id (), radius, stix_get_unit_name (length_unit));

    RoseDesign *des = slot->getRootObject()->design(); 
    Woodruff_slot_end_type *type = Woodruff_slot_end_type::newInstance(des);
    type->put_radius (stix_make_length_measure_item (des, radius, length_unit));
    slot->put_start_condition (type->getRoot());
    return slot;
}

Slot *set_woodruff_slot_end_type_at_end (Slot *slot, double radius, StixUnit length_unit) 
{
    if (slot == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set woodruff slot end type at end: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set slot type at end for slot %s to woodruff end type with radius %g (%s)", slot->get_its_id (), radius, stix_get_unit_name (length_unit));

    RoseDesign *des = slot->getRootObject()->design(); 
    Woodruff_slot_end_type *type = Woodruff_slot_end_type::newInstance(des);
    type->put_radius (stix_make_length_measure_item (des, radius, length_unit));
    slot->put_end_condition (type->getRoot());
    return slot;
}

Slot *set_flat_slot_end_type_at_start (Slot *slot, double corner_radius_1, double corner_radius_2, StixUnit length_unit) 
{
    if (slot == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set flat slot end type at start: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set slot type at start for slot %s to flat end type with radii (%g, %g) (%s)", slot->get_its_id (), corner_radius_1, corner_radius_2, stix_get_unit_name (length_unit));

    RoseDesign *des = slot->getRootObject()->design(); 
    Flat_slot_end_type *type = Flat_slot_end_type::newInstance(des);
    type->put_corner_radius1 (stix_make_length_measure_item (des, corner_radius_1, length_unit));
    type->put_corner_radius2 (stix_make_length_measure_item (des, corner_radius_2, length_unit));
    slot->put_start_condition (type->getRoot());
    return slot;
}

Slot *set_flat_slot_end_type_at_end (Slot *slot, double corner_radius_1, double corner_radius_2, StixUnit length_unit) 
{
    if (slot == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set flat slot end type at end: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set slot type at end for slot %s to flat end type with radii (%g, %g) (%s)", slot->get_its_id (), corner_radius_1, corner_radius_2, stix_get_unit_name (length_unit));

    RoseDesign *des = slot->getRootObject()->design(); 
    Flat_slot_end_type *type = Flat_slot_end_type::newInstance(des);
    type->put_corner_radius1 (stix_make_length_measure_item (des, corner_radius_1, length_unit));
    type->put_corner_radius2 (stix_make_length_measure_item (des, corner_radius_2, length_unit));
    slot->put_end_condition (type->getRoot());
    return slot;
}

Step *new_step (Workpiece *piece, char * nme,  StixMtrx placement, double length, StixUnit length_unit)
{
    if (piece == NULL || length_unit == NULL) {
	REPORT_ERROR ("New step: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New step for workpiece %s name= %s,  profile length = %g length unit = %s", piece->get_its_id (), nme, length, 
			stix_get_unit_name (length_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Step * step = Step::newInstance (des);
    step->put_feature_placement (placement.makeAp3dIn (des));
    step->put_its_id (nme);
    step->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, length, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    step->put_open_boundary (path->getRoot());

    return step;
}

Step *set_open_vee_wall (Step *step, double profile_radius, double profile_angle, double tilt_angle, double first_side_length, double second_side_length, StixUnit length_unit, StixUnit angle_unit)
{
    if (step == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("Set open vee wall: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Set open vee wall for step %s profile_radius = %g profile_angle = %g tilt_angle = %g, first_side_length = %g second_side_length = %g length_unit=%s, angle_unit=%s", 
		step->get_its_id (), profile_radius, profile_angle, tilt_angle, first_side_length, second_side_length, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = step->getRootObject()->design();    
    
    Vee_profile * profile =  Vee_profile::newInstance (des);
    profile->put_profile_radius (stix_make_length_measure_item (des, profile_radius, length_unit));
    profile->put_first_side_length (stix_make_length_measure_item (des, first_side_length, length_unit));
    profile->put_second_side_length (stix_make_length_measure_item (des, second_side_length, length_unit));
    profile->put_profile_angle (stix_make_angle_measure_item (des, profile_angle, angle_unit));
    profile->put_tilt_angle (stix_make_angle_measure_item (des, tilt_angle, angle_unit));

    profile->put_placement (StixMtrx().makeAp3dIn (des));
    step->put_wall_boundary(profile->getRoot());

    return step;
}

Spherical_cap *new_spherical_cap (Workpiece *piece, char *nme, StixMtrx placement, double radius, double internal_angle, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("New spherical cap: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New spherical cap for workpiece %s name= %s, radius = %g (%s) internal_angle = %g (%s)", piece->get_its_id (), nme, radius, stix_get_unit_name (length_unit), internal_angle, angle_unit);

    RoseDesign *des = piece->getRootObject()->design();    
    
    Spherical_cap * cap = Spherical_cap::newInstance (des);
    cap->put_feature_placement (placement.makeAp3dIn (des));
    cap->put_its_id (nme);
    cap->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    cap->put_radius (stix_make_length_measure_item (des, radius, length_unit));
    cap->put_internal_angle (stix_make_angle_measure_item (des, radius, angle_unit));

    return cap;
}

Rounded_end *new_rounded_end (Workpiece *piece, char * nme,  StixMtrx placement, double end_depth, double radius, double swept_angle, StixUnit length_unit, StixUnit angle_unit)
{
    if (piece == NULL || length_unit == NULL || angle_unit == NULL) {
	REPORT_ERROR ("New rounded end: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("New rounded end for workpiece %s name= %s, pocket depth = %g radius = %g sweep = %g length unit = %s, angle unit = %s ", piece->get_its_id (), nme, end_depth, radius, 
			swept_angle, stix_get_unit_name (length_unit), stix_get_unit_name (angle_unit));

    RoseDesign *des = piece->getRootObject()->design();    
    
    Rounded_end * end = Rounded_end::newInstance (des);
    end->put_feature_placement (placement.makeAp3dIn (des));
    end->put_its_id (nme);
    end->put_its_workpiece (piece->getRoot());

    Linear_path * path = Linear_path::newInstance (des);
    path->put_distance (stix_make_length_measure_item (des, end_depth, length_unit));
    path->put_placement (StixMtrx().makeAp3dIn (des));
    end->put_course_of_travel (path->getRoot());

    Partial_circular_profile * profile =  Partial_circular_profile::newInstance (des);
    profile->put_radius (stix_make_length_measure_item (des, radius, length_unit));
    profile->put_sweep_angle (stix_make_angle_measure_item (des, swept_angle, angle_unit));
    profile->put_placement (StixMtrx().makeAp3dIn (des));
    end->put_partial_circular_boundary(profile->getRoot());

    return end;
}

// Items not yet in the ARM
// Defined_thread *new_defined_thread (Machining_feature_IF *first_feature, char * nme,  StixMtrx placement, const char *inner_or_outer, const char *fit_class, const char *form, double major_diameter, 
//				int number_of_threads, const char * thread_hand, double effective_length, double pitch_diameter);

// ngon closed pocket
// ngon 
