#ifndef FEATUREMAKER_H
#define FEATUREMAKER_H

// My report is flag used to control the reporting of errors in all the makers
// Errors can be reported to a message box, a trace file, both or neither
#ifndef MY_REPORT
#define MY_REPORT
#define REPORT_ERROR(...) featuremaker_report_error(__VA_ARGS__)
// Microsoft code for the message boxes called by REPORT_ERROR - comment out if not wanted.
// If not wanted must also change toolmaker_report_error code to not call the boxes
#include <windows.h>			
#endif

// Tracing of function calls for this maker can be eliminated by selecting appropriate definition for TRACE_CALL
#ifndef MY_TRACE
#define MY_TRACE
#ifndef TRACE_CALL	// select one
#define TRACE_CALL featuremaker_function_trace
//#define TRACE_CALL(...) 
#endif
#endif

// STIX defines functions to handle units and other useful utilities
#include <stp_schema.h>
#include <stix.h>

// ARM defines all the Application Objects in the ARM library (long list)
#include "ARM.h"

// File used by all the makers for tracing - must be defined in main program
extern FILE *pfLog;

// function calls are sent to a trace file or eliminated from code by choosing appropriate define for TRACE_FILE
void featuremaker_function_trace(const char * fmt, ...);

// Errors are reported to message box, the function_trace file, both or neither
void featuremaker_report_error(const char * fmt, ...);

// Functions define the location and direction of a feature by creating a StixMtrx that is then converted to an stp_axis2_placement_3d in the new_feature functions
StixMtrx new_placement (double x, double y, double z, double i, double j, double k, double a, double b, double c);
StixMtrx new_simple_placement (double x, double y, double z);

// Generic feature for when a more specific type is not available, or when the feature only has to be defined by its faces
Toolpath_feature    *	new_toolpath_feature (Workpiece *piece, char * nme, StixMtrx placement);

Round_hole	    *	new_round_hole (Workpiece *piece, char * nme, double depth, double diameter, StixMtrx placement, StixUnit length_unit);
Round_hole	    *	set_hole_though_bottom (Round_hole *hole);
Round_hole	    *	set_hole_flat_bottom (Round_hole *hole);
Round_hole	    *	set_hole_conical_bottom (Round_hole * hole, double tip_angle, double tip_radius, StixUnit angle_unit, StixUnit length_unit);
Round_hole	    *	set_hole_spherical_bottom (Round_hole *hole, double radius, StixUnit length_unit);
Round_hole	    *	set_hole_flat_with_radius_bottom (Round_hole *hole, double corner_radius, StixUnit length_unit);

Round_hole	    *	set_hole_angle_taper (Round_hole *hole, double angle, StixUnit angle_unit);
Round_hole	    *	set_hole_final_diameter_taper (Round_hole *hole, double final_diameter, StixUnit length_unit);

Closed_pocket	    *	new_closed_rectangular_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double length, double width, StixUnit length_unit);
Closed_pocket	    *	new_closed_circular_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double diameter, StixUnit length_unit);

Closed_pocket	    *	new_closed_path_pocket (Workpiece *piece, char * nme,  StixMtrx placement, stp_bounded_curve *curve, double depth, StixUnit length_unit);
stp_bounded_curve   *	start_profile_curve (RoseDesign *des, double x, double y, double z);
stp_bounded_curve   *	profile_curve_add_xyz (stp_bounded_curve *curve, double x, double y, double z);
stp_bounded_curve   *	profile_curve_add_arc (stp_bounded_curve *curve, double x, double y, double z);

Pocket_IF	    *	set_pocket_orthogonal_radius (Pocket_IF *pocket, double orthogonal_radius, StixUnit length_unit);
Pocket_IF	    *	set_pocket_planar_radius (Pocket_IF *pocket, double planar_radius, StixUnit length_unit);
Pocket_IF	    *	set_pocket_slope (Pocket_IF *pocket, double slope, StixUnit angle_unit);

Pocket_IF	    *	set_pocket_through_bottom (Pocket_IF *pocket);
Pocket_IF	    *	set_pocket_flat_bottom (Pocket_IF *pocket);
Pocket_IF	    *	set_pocket_radiused_bottom (Pocket_IF *pocket, double radius, double x, double y, double z, StixUnit length_unit);

Open_pocket	    *	new_open_circular_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double radius, double swept_angle, StixUnit length_unit, StixUnit angle_unit);
Open_pocket	    *	new_open_linear_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double length, StixUnit length_unit);
Open_pocket	    *	new_open_rounded_u_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double u_width, double u_depth, StixUnit length_unit);
Open_pocket	    *	new_open_square_u_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double u_width, double u_first_radius, double u_first_angle, 
			    double u_second_radius, double u_second_angle, double u_depth, StixUnit length_unit, StixUnit angle_unit);
/* Tee_profile currently missing from ARM library
Open_pocket *new_open_tee_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double depth, double u_width, double u_first_angle, double u_second_angle, 
	    double cross_bar_width, double cross_bar_depth, double u_radius, double u_depth, double first_offset, double second_offset, StixUnit length_unit, StixUnit angle_unit); */
Open_pocket	    *   new_open_vee_pocket (Workpiece *piece, char * nme,  StixMtrx placement, double pocket_depth, double profile_radius, double profile_angle, double tilt_angle,
			    double first_side_length, double second_side_length, StixUnit length_unit, StixUnit angle_unit);
Open_pocket	    *	new_open_path_pocket (Workpiece *piece, char * nme,  StixMtrx placement, stp_bounded_curve *curve, double pocket_depth, StixUnit length_unit);

// Wall describes boundary of material before pocket removed
Open_pocket	    *	set_open_circular_wall (Open_pocket *pocket, double radius, double swept_angle, StixUnit length_unit, StixUnit angle_unit);
Open_pocket	    *	set_open_linear_wall (Open_pocket *pocket, double length, StixUnit length_unit);
Open_pocket	    *	set_open_rounded_u_wall (Open_pocket *pocket, char * nme, double u_width, double u_depth, StixUnit length_unit);
Open_pocket	    *	set_open_square_u_wall (Open_pocket *pocket, double u_width, double u_first_radius, double u_first_angle, double u_second_radius, double u_second_angle, double u_depth, StixUnit length_unit, StixUnit angle_unit);
/* Tee_profile currently missing from ARM library
Open_pocket *set_open_tee_wall (Open_pocket *pocket, double u_width, double u_first_angle, double u_second_angle, 
	    double cross_bar_width, double cross_bar_depth, double u_radius, double u_depth, double first_offset, double second_offset, StixUnit length_unit, StixUnit angle_unit); */
Open_pocket	    *	set_open_vee_wall (Open_pocket *pocket, double profile_radius, double profile_angle, double tilt_angle, double first_side_length, double second_side_length, StixUnit length_unit, StixUnit angle_unit);
Open_pocket	    *	set_open_path_wall (Open_pocket *pocket, stp_bounded_curve *curve);

Pocket_IF	    *	add_boss_to_pocket (Pocket_IF *pocket, Boss_IF *boss);

Rectangular_boss    *	new_rectangular_boss (Workpiece *piece, char * nme, StixMtrx placement, double height, double length, double width, StixUnit length_unit);
Circular_boss	    *	new_circular_boss (Workpiece *piece, char * nme, StixMtrx placement, double height, double diameter, StixUnit length_unit);
Complex_boss	    *	new_path_boss (Workpiece *piece, char * nme, StixMtrx placement, double height, stp_bounded_curve *curve, StixUnit length_unit);
Boss_IF		    *	set_boss_slope (Boss_IF *boss, double slope, StixUnit angle_unit);

Planar_face	    *   new_planar_face (Workpiece *piece, char * nme, StixMtrx placement, double depth, double length, double width, StixUnit length_unit);
Planar_face	    *	add_boss_to_face (Planar_face *face, Boss_IF *boss);
Planar_face	    *	set_face_circular_boundary (Planar_face *face, StixMtrx placement, double diameter, StixUnit length_unit);
Planar_face	    *	set_face_rectangular_boundary (Planar_face *face, StixMtrx placement, double length, double width, StixUnit length_unit);
Planar_face	    *	set_face_path_boundary (Planar_face *face, StixMtrx placement, stp_bounded_curve *curve);

// Outside profile features are different to shape profile ones because they have no floor
General_outside_profile		    *	new_closed_rectangular_outside_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, double length, double width, StixUnit length_unit);
General_outside_profile		    *	new_closed_circular_outside_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, double diameter, StixUnit length_unit);
General_outside_profile		    *	new_partial_circular_outside_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, double radius, double sweep_angle, StixUnit length_unit, StixUnit angle_unit);
General_outside_profile		    *	new_linear_outside_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, double length, StixUnit length_unit);
General_outside_profile		    *	new_closed_path_outside_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, stp_bounded_curve *curve, StixUnit length_unit);
General_outside_profile		    *	new_open_path_outside_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, stp_bounded_curve *curve, StixUnit length_unit);

Rectangular_closed_shape_profile    *	new_rectangular_closed_shape_profile (Workpiece *piece, char * nme,  StixMtrx placement, StixVctr removal_direction, double feature_depth, double length, double width, StixUnit length_unit);
Circular_closed_shape_profile	    *	new_circular_closed_shape_profile (Workpiece *piece, char * nme,  StixMtrx placement, StixVctr removal_direction, double feature_depth, double diameter, StixUnit length_unit);
Partial_circular_shape_profile	    *	new_partial_circular_shape_profile (Workpiece *piece, char * nme,  StixMtrx placement, StixVctr removal_direction, double feature_depth, double radius, double sweep_angle, StixUnit length_unit, StixUnit angle_unit);
Rectangular_open_shape_profile	    *	new_rectangular_open_shape_profile (Workpiece *piece, char * nme,  StixMtrx placement, StixVctr removal_direction, double feature_depth, double u_width, double u_first_radius, double u_first_angle, 
					    double u_second_radius, double u_second_angle, double u_depth, StixUnit length_unit, StixUnit angle_unit);
General_shape_profile		    *	new_closed_path_shape_profile (Workpiece *piece, char * nme,  StixMtrx placement, StixVctr removal_direction, double feature_depth, stp_bounded_curve *curve, StixUnit length_unit);
General_shape_profile		    *	new_open_path_shape_profile (Workpiece *piece, char * nme,  StixMtrx placement, double feature_depth, stp_bounded_curve *curve, StixUnit length_unit);

// Floors for shape profile features
Planar_profile_floor		    *	new_planar_profile_floor (Workpiece *piece, stp_plane *plane, const char *start_or_end, double floor_radius, StixUnit length_unit);
General_profile_floor		    *	new_general_profile_floor (Workpiece *piece, stp_face *face, const char *start_or_end, double floor_radius, StixUnit length_unit);
Through_profile_floor		    *	new_through_profile_floor (Workpiece *piece);
Shape_profile_IF		    *	set_shape_profile_floor (Shape_profile_IF *feature, Profile_floor_IF *floor);

// Slots 
Slot		    *	new_open_circular_slot (Workpiece *piece, char * nme,  StixMtrx placement, double slot_depth, double radius, double swept_angle, StixUnit length_unit, StixUnit angle_unit);
Slot		    *	new_open_linear_slot (Workpiece *piece, char * nme,  StixMtrx placement, double slot_depth, double length, StixUnit length_unit);
Slot		    *	new_open_rounded_u_slot (Workpiece *piece, char * nme,  StixMtrx placement, double slot_depth, double u_width, double u_depth, StixUnit length_unit);
Slot		    *	new_open_square_u_slot (Workpiece *piece, char * nme,  StixMtrx placement, double slot_depth, double u_width, double u_first_radius, double u_first_angle, 
						double u_second_radius, double u_second_angle, double u_depth, StixUnit length_unit, StixUnit angle_unit);
/* Tee_profile currently missing from ARM library
Slot		    *	new_open_tee_slot (Workpiece *piece, char * nme,  StixMtrx placement, double slot_depth, double u_width, double u_first_angle, double u_second_angle, 
					    double cross_bar_width, double cross_bar_depth, double u_radius, double u_depth, double first_offset, double second_offset, StixUnit length_unit, StixUnit angle_unit); */
Slot		    *	new_open_vee_slot (Workpiece *piece, char * nme,  StixMtrx placement, double slot_depth, double profile_radius, double profile_angle, double tilt_angle,
						double first_side_length, double second_side_length, StixUnit length_unit, StixUnit angle_unit);
Slot		    *	new_open_path_slot (Workpiece *piece, char * nme,  StixMtrx placement, stp_bounded_curve *curve, double slot_depth, StixUnit length_unit);

Slot		    *	set_open_slot_end_type_at_start (Slot *slot);
Slot		    *	set_open_slot_end_type_at_end (Slot *slot);
Slot		    *	set_radiused_slot_end_type_at_start (Slot *slot);
Slot		    *	set_radiused_slot_end_type_at_end (Slot *slot);
Slot		    *	set_loop_slot_end_type_at_start (Slot *slot);
Slot		    *	set_loop_slot_end_type_at_end (Slot *slot);
Slot		    *	set_woodruff_slot_end_type_at_start (Slot *slot, double radius, StixUnit length_unit);
Slot		    *	set_woodruff_slot_end_type_at_end (Slot *slot, double radius, StixUnit length_unit);
Slot		    *	set_flat_slot_end_type_at_start (Slot *slot, double corner_radius_1, double corner_radius_2, StixUnit length_unit);
Slot		    *	set_flat_slot_end_type_at_end (Slot *slot, double corner_radius_1, double corner_radius_2, StixUnit length_unit);

Step		    *	new_step (Workpiece *piece, char * nme,  StixMtrx placement, double length, StixUnit length_unit);
Step		    *	add_boss_to_step (Step *step, Boss_IF *boss);
Step		    *	set_open_vee_wall (Step *step, double profile_radius, double profile_angle, double tilt_angle, double first_side_length, double second_side_length, StixUnit length_unit, StixUnit angle_unit);

Spherical_cap	    *	new_spherical_cap (Workpiece *piece, char *nme, StixMtrx placement, double radius, double internal_angle, StixUnit length_unit, StixUnit angle_unit);
Rounded_end	    *	new_rounded_end (Workpiece *piece, char * nme,  StixMtrx placement, double end_depth, double radius, double swept_angle, StixUnit length_unit, StixUnit angle_unit);

// Not in ARM yet
//Defined_thread	    *   new_defined_thread (Machining_feature *first_feature, char * nme,  StixMtrx placement, const char *inner_or_outer, const char *fit_class, const char *form, double major_diameter, 
//				int number_of_threads, const char * thread_hand, double effective_length, double pitch_diameter);

// Faces can be added to all the different types of features but most commonly they are added to toolpath features
Manufacturing_feature_IF *add_face_to_feature (Manufacturing_feature_IF *feature, stp_advanced_face *face);

#endif