
#ifdef MASTERCAM_AFX
#include "stdafx.h"
#endif

#include "ToolMaker.h"

// Calls to this function can be eliminated from code by unsetting #MY_TRACE in ToolMaker.h
void toolmaker_function_trace(const char * fmt, ...)
{
#ifdef MY_TRACE
    if (pfLog == NULL) 
	return;
    va_list ap;
    va_start(ap, fmt);

    char buff[10240];
    char * c = buff;
    vsprintf (c, fmt, ap);

    va_end(ap);

    static int count = 0;
//    printf ("Tolerance_maker Call#%d: %s\n", count, message);
    fprintf (pfLog, "Tool_maker\tCall#%d: %s\n", count, buff);
    count++;   
#endif
}

// Error reports can go to a message box, the trace file or nowhere depending on setting of #MY_REPORT and #MY_TRACE
void toolmaker_report_error(const char * fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);

    char buff[10240];
    char * c = buff;
    vsprintf (c, fmt, ap);

    va_end(ap);

	// Windows gadget to show a warning box requires windows.h
#ifdef MY_REPORT
    MessageBox (NULL, buff, "WARNING", MB_ICONSTOP);
#endif
#ifdef MY_TRACE
    if (pfLog == NULL) 
	return;
fprintf (pfLog, "Tool_maker\tERROR: %s\n", buff);
#endif
}


// User_defined tool for APT processors - CAM systems should use more specific kinds of tools
User_defined_milling_tool *new_apt_tool (RoseDesign *des, const char * id, double tool_diameter, double tool_radius, 
	double horizontal_distance, double vertical_distance, double unused, double angle, double height, StixUnit length_unit, StixUnit angle_unit)
{
    TRACE_CALL ("new apt tool id = %s, diameter = %g radius = %g", id, tool_diameter, tool_radius);
    if (des == NULL) {
	REPORT_ERROR ("User defined milling tool: function called with NULL arguments");
	return NULL;
    }

    User_defined_milling_tool *tool = User_defined_milling_tool::newInstance (des);
    tool->put_its_id (id);

    tool->put_effective_cutting_diameter (stix_make_measure_item (des, tool_diameter, length_unit));
    tool->put_corner_radius (stix_make_measure_item (des, tool_radius, length_unit));
    tool->put_corner_radius_center_horizontal (stix_make_measure_item (des, horizontal_distance, length_unit));
    tool->put_corner_radius_center_vertical (stix_make_measure_item (des, vertical_distance, length_unit));
    tool->put_tip_outer_angle (stix_make_measure_item (des, unused, angle_unit));
    tool->put_taper_angle (stix_make_measure_item (des, angle, angle_unit));
    tool->put_overall_assembly_length (stix_make_measure_item (des,  height, length_unit));
    tool->put_maximum_depth_of_cut (stix_make_measure_item (des, height,length_unit));

    return tool;
}


Endmill *new_endmill_tool (RoseDesign *des, const char * id, double tool_diameter, double overall_assembly_length, double functional_length, double max_depth_of_cut, 
						    double num_effective_teeth, double cutting_edge_angle, StixUnit length_unit,  StixUnit angle_unit)
{
    TRACE_CALL ("new endmill tool id = %s, diameter = %g max_depth_of_cut = %g", id, tool_diameter, max_depth_of_cut);
    if (des == NULL) {
	REPORT_ERROR ("Endmill tool: function called with NULL arguments");
	return NULL;
    }

    Endmill *tool = Endmill::newInstance (des);
    tool->put_its_id (id);

    tool->put_effective_cutting_diameter (stix_make_measure_item (des, tool_diameter, length_unit));
    tool->put_overall_assembly_length (stix_make_measure_item (des,  overall_assembly_length, length_unit));
    tool->put_tool_cutting_edge_angle (stix_make_measure_item (des, cutting_edge_angle, angle_unit));
    tool->put_maximum_depth_of_cut (stix_make_measure_item (des, max_depth_of_cut ,length_unit));	
    tool->put_number_of_effective_teeth (num_effective_teeth);

    Cutting_component *comp = Cutting_component::newInstance (des);
    comp->put_tool_functional_length (stix_make_measure_item (des, functional_length ,length_unit));
    tool->add_its_cutting_edges (comp->getRoot ());

    return tool;
}

Facemill *new_facemill_tool (RoseDesign *des, const char * id, double tool_diameter, double overall_assembly_length, double functional_length, double max_depth_of_cut, 
						    double num_effective_teeth, double cutting_edge_angle, StixUnit length_unit, StixUnit angle_unit)
{
    TRACE_CALL ("new facemill tool id = %s, diameter = %g max_depth_of_cut = %g", id, tool_diameter, max_depth_of_cut);
    if (des == NULL) {
	REPORT_ERROR ("Facemill tool: function called with NULL arguments");
	return NULL;
    }

    Facemill *tool = Facemill::newInstance (des);
    tool->put_its_id (id);

    tool->put_effective_cutting_diameter (stix_make_measure_item (des, tool_diameter, length_unit));
    tool->put_overall_assembly_length (stix_make_measure_item (des,  overall_assembly_length, length_unit));
    tool->put_tool_cutting_edge_angle (stix_make_measure_item (des, cutting_edge_angle, angle_unit));
    tool->put_maximum_depth_of_cut (stix_make_measure_item (des, max_depth_of_cut ,length_unit));	
    tool->put_number_of_effective_teeth (num_effective_teeth);

    Cutting_component *comp = Cutting_component::newInstance (des);
    comp->put_tool_functional_length (stix_make_measure_item (des, functional_length ,length_unit));
    tool->add_its_cutting_edges (comp->getRoot ());

    return tool;
}

Tapping_cutting_tool *new_tapping_tool (RoseDesign *des, const char * id, double tool_diameter, double overall_assembly_length, double functional_length, double max_depth_of_cut, 
						    const char *thread_form_type, double pitch, double size, StixUnit length_unit)
{
    TRACE_CALL ("new tapping tool id = %s, diameter = %g max_depth_of_cut = %g", id, tool_diameter, max_depth_of_cut);
    if (des == NULL) {
	REPORT_ERROR ("Tapping tool: function called with NULL arguments");
	return NULL;
    }

    Tapping_cutting_tool *tool = Tapping_cutting_tool::newInstance (des);
    tool->put_its_id (id);

    tool->put_effective_cutting_diameter (stix_make_measure_item (des, tool_diameter, length_unit));
    tool->put_overall_assembly_length (stix_make_measure_item (des,  overall_assembly_length, length_unit));
     tool->put_maximum_depth_of_cut (stix_make_measure_item (des, max_depth_of_cut ,length_unit));	

    tool->put_thread_size (stix_make_measure_item (des, size, length_unit));
    tool->put_thread_pitch (stix_make_measure_item (des, pitch, length_unit));				    // Why length measure?
    tool->put_thread_form_type (thread_form_type);

    Cutting_component *comp = Cutting_component::newInstance (des);
    comp->put_tool_functional_length (stix_make_measure_item (des, functional_length ,length_unit));
    tool->add_its_cutting_edges (comp->getRoot ());

    return tool;
}

Spotdrill *new_spot_drill (RoseDesign *des, const char * id, double tool_diameter, double overall_assembly_length, double functional_length, double max_depth_of_cut, 
						    double point_angle, StixUnit length_unit, StixUnit angle_unit)
{
    TRACE_CALL ("new spot drill id = %s, diameter = %g max_depth_of_cut = %g point_angle = %g", id, tool_diameter, max_depth_of_cut, point_angle);
    if (des == NULL) {
	REPORT_ERROR ("Spotdrill tool: function called with NULL arguments");
	return NULL;
    }

    Spotdrill *tool = Spotdrill::newInstance (des);
    tool->put_its_id (id);

    tool->put_effective_cutting_diameter (stix_make_measure_item (des, tool_diameter, length_unit));
    tool->put_overall_assembly_length (stix_make_measure_item (des,  overall_assembly_length, length_unit));
    tool->put_point_angle (stix_make_measure_item (des, point_angle, angle_unit));
    tool->put_maximum_depth_of_cut (stix_make_measure_item (des, max_depth_of_cut ,length_unit));	

    Cutting_component *comp = Cutting_component::newInstance (des);
    comp->put_tool_functional_length (stix_make_measure_item (des, functional_length ,length_unit));
    tool->add_its_cutting_edges (comp->getRoot ());

    return tool;
}

Twist_drill *new_twist_drill (RoseDesign *des, const char * id, double tool_diameter, double overall_assembly_length, double functional_length, double max_depth_of_cut, 
						    double point_angle, StixUnit length_unit, StixUnit angle_unit)
{
    TRACE_CALL ("new twist drill id = %s, diameter = %g max_depth_of_cut = %g point_angle = %g", id, tool_diameter, max_depth_of_cut, point_angle);
    if (des == NULL) {
	REPORT_ERROR ("Twistdrill tool: function called with NULL arguments");
	return NULL;
    }

    Twist_drill *tool = Twist_drill::newInstance (des);
    tool->put_its_id (id);

    tool->put_effective_cutting_diameter (stix_make_measure_item (des, tool_diameter, length_unit));
    tool->put_overall_assembly_length (stix_make_measure_item (des,  overall_assembly_length, length_unit));
    tool->put_point_angle (stix_make_measure_item (des, point_angle, angle_unit));
    tool->put_maximum_depth_of_cut (stix_make_measure_item (des, max_depth_of_cut ,length_unit));	

    Cutting_component *comp = Cutting_component::newInstance (des);
    comp->put_tool_functional_length (stix_make_measure_item (des, functional_length ,length_unit));
    tool->add_its_cutting_edges (comp->getRoot ());

    return tool;
}

Rotating_boring_cutting_tool *new_boring_tool (RoseDesign *des, const char * id, double tool_diameter, double overall_assembly_length, double functional_length, double max_depth_of_cut, 
						    StixUnit length_unit)
{
    TRACE_CALL ("new boring tool id = %s, diameter = %g max_depth_of_cut = %g", id, tool_diameter, max_depth_of_cut);
    if (des == NULL) {
	REPORT_ERROR ("Boring tool: function called with NULL arguments");
	return NULL;
    }

    Rotating_boring_cutting_tool *tool = Rotating_boring_cutting_tool::newInstance (des);
    tool->put_its_id (id);

    tool->put_effective_cutting_diameter (stix_make_measure_item (des, tool_diameter, length_unit));
    tool->put_overall_assembly_length (stix_make_measure_item (des,  overall_assembly_length, length_unit));
    tool->put_maximum_depth_of_cut (stix_make_measure_item (des, max_depth_of_cut ,length_unit));	

    Cutting_component *comp = Cutting_component::newInstance (des);
    comp->put_tool_functional_length (stix_make_measure_item (des, functional_length ,length_unit));
    tool->add_its_cutting_edges (comp->getRoot ());

    return tool;
}

Machining_tool_IF *set_tool_usage (Machining_tool_IF *tool, Workpiece *piece)
{
    if (tool == NULL || piece == NULL) {
	REPORT_ERROR ("Set tool usage: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Geometry model for tool %s set to workpiece %s", tool->get_its_id(), piece->get_its_id());

    RoseDesign *des = tool->getRootObject ()->design();

    Tool_usage *use = Tool_usage::find (tool->get_its_usage());
    if (use == NULL) {
	use = Tool_usage::newInstance (des);
    }

    use->put_its_product (piece->getRoot ());

    return tool;
}

Machining_tool_IF *set_tool_carousel (Machining_tool_IF *tool, const char * carousel)
{
    if (tool == NULL || carousel == NULL) {
	REPORT_ERROR ("Set tool carousel: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Carousel for tool %s set to %s", tool->get_its_id(), carousel);

    RoseDesign *des = tool->getRootObject ()->design();

    Tool_usage *use = Tool_usage::find (tool->get_its_usage());
    if (use == NULL) {
	use = Tool_usage::newInstance (des);
    }

    use->put_its_carousel (carousel);

    return tool;
}


Machining_tool_IF *set_tool_carousel_position (Machining_tool_IF *tool, const char *position)
{
    if (tool == NULL || position == NULL) {
	REPORT_ERROR ("Set tool carousel position: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Carousel position for tool %s set to %s", tool->get_its_id(), position);

    RoseDesign *des = tool->getRootObject ()->design();

    Tool_usage *use = Tool_usage::find (tool->get_its_usage());
    if (use == NULL) {
	use = Tool_usage::newInstance (des);
    }

    use->put_its_carousel (position);

    return tool;
}

Machining_tool_IF *set_tool_end_placement (Machining_tool_IF *tool, StixMtrx M)
{
    if (tool == NULL) {
	REPORT_ERROR ("Set tool end placement: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("End placement for tool %s set to (%g, %g, %g) z=(%g, %g, %g) x=(%g, %g, %g)", tool->get_its_id (), M.w().get(0), M.w().get(1), M.w().get(2), M.z().get(0), M.z().get(1), M.z().get(2), M.x().get(0), M.x().get(1), M.x().get(2));;

    RoseDesign *des = tool->getRootObject ()->design();

    Tool_usage *use = Tool_usage::find (tool->get_its_usage());
    if (use == NULL) {
	use = Tool_usage::newInstance (des);
    }

    use->put_tool_end_placement (M.makeAp3dIn (des));

    return tool;
}

Machining_tool_IF *set_tool_gage_line_placement (Machining_tool_IF *tool, StixMtrx M)
{
    if (tool == NULL) {
	REPORT_ERROR ("Set tool gage line placement: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Gage line placement for tool %s set to (%g, %g, %g) z=(%g, %g, %g) x=(%g, %g, %g)", tool->get_its_id (), M.w().get(0), M.w().get(1), M.w().get(2), M.z().get(0), M.z().get(1), M.z().get(2), M.x().get(0), M.x().get(1), M.x().get(2));;

    RoseDesign *des = tool->getRootObject ()->design();

    Tool_usage *use = Tool_usage::find (tool->get_its_usage());
    if (use == NULL) {
	use = Tool_usage::newInstance (des);
    }

    use->put_gage_placement (M.makeAp3dIn (des));

    return tool;
}

Milling_machine_cutting_tool_IF	*set_tool_expected_life (Milling_machine_cutting_tool_IF *tool, double life, StixUnit time_unit, unsigned component_index)
{
    if (tool == NULL) {
	REPORT_ERROR ("Set tool expected life: function called with NULL arguments");
	return NULL;
    }
    if (component_index != 0 && component_index > tool->size_its_cutting_edges ()) {
	REPORT_ERROR ("Set tool expected life for tool %s: component index %d out of range [0, %d]", tool->get_its_id(), component_index, tool->size_its_cutting_edges () - 1);
	return NULL;
    }
    TRACE_CALL ("Expected life for tool %s component %d set to %g (%s)", tool->get_its_id (), component_index, life, stix_get_unit_name (time_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    if (component_index >=  tool->size_its_cutting_edges ()) {
	Cutting_component *component = Cutting_component::newInstance (des);
	tool->add_its_cutting_edges (component->getRoot());
    }

    Cutting_component *component = Cutting_component::find (tool->get_its_cutting_edges (component_index)->getValue());
//    component->put_expected_tool_life (stix_make_measure_item (des, life, time_unit));

    return tool; 
}

Milling_machine_cutting_tool_IF	*set_tool_cutting_material (Milling_machine_cutting_tool_IF *tool, const char *material, const char *material_standard, unsigned component_index)
{
    if (tool == NULL) {
	REPORT_ERROR ("Set tool material: function called with NULL arguments");
	return NULL;
    }
    if (component_index != 0 && component_index > tool->size_its_cutting_edges ()) {
	REPORT_ERROR ("Set material for tool %s: component index %d out of range [0, %d]", tool->get_its_id(), component_index, tool->size_its_cutting_edges () - 1);
	return NULL;
    }
    TRACE_CALL ("Material for tool %s component %d set to %s standard %s", tool->get_its_id (), component_index, material, material_standard);

    RoseDesign *des = tool->getRootObject ()->design();

    if (component_index >=  tool->size_its_cutting_edges ()) {
	Cutting_component *component = Cutting_component::newInstance (des);
	tool->add_its_cutting_edges (component->getRoot());
    }

    Cutting_component *component = Cutting_component::find (tool->get_its_cutting_edges (component_index)->getValue());

    Material * mat = Material::find (component->get_its_material());
    if (mat == NULL) {
	mat = Material::newInstance (des);
	component->put_its_material (mat->getRoot());
    }

    mat->put_material_identifier (material);
    mat->put_standard_identifier (material_standard);

    return tool;
}

Milling_machine_cutting_tool_IF	*set_tool_functional_length (Milling_machine_cutting_tool_IF *tool, double length, StixUnit length_unit, unsigned component_index)
{
    if (tool == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set tool functional length: function called with NULL arguments");
	return NULL;
    }
    if (component_index != 0 && component_index > tool->size_its_cutting_edges ()) {
	REPORT_ERROR ("Set tool functional length for tool %s: component index %d out of range [0, %d]", tool->get_its_id(), component_index, tool->size_its_cutting_edges () - 1);
	return NULL;
    }
    TRACE_CALL ("Functional length for tool %s component %d set to %g (%s)", tool->get_its_id (), component_index, length, stix_get_unit_name (length_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    if (component_index >=  tool->size_its_cutting_edges ()) {
	Cutting_component *component = Cutting_component::newInstance (des);
	tool->add_its_cutting_edges (component->getRoot());
    }

    Cutting_component *component = Cutting_component::find (tool->get_its_cutting_edges (component_index)->getValue());
    component->put_tool_functional_length (stix_make_length_measure_item (des, length, length_unit));

    return tool;
}

Milling_machine_cutting_tool_IF	*set_tool_recommended_speed (Milling_machine_cutting_tool_IF *tool, double spindle_speed, StixUnit speed_unit, unsigned component_index)
{
    if (tool == NULL || speed_unit == NULL) {
	REPORT_ERROR ("Set tool recommended speed: function called with NULL arguments");
	return NULL;
    }
    if (component_index != 0 && component_index > tool->size_its_cutting_edges ()) {
	REPORT_ERROR ("Set tool recommended speed for tool %s: component index %d out of range [0, %d]", tool->get_its_id(), component_index, tool->size_its_cutting_edges () - 1);
	return NULL;
    }
    TRACE_CALL ("Recommended speed for tool %s component %d set to %g (%s)", tool->get_its_id (), component_index, spindle_speed, stix_get_unit_name (speed_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    if (component_index >=  tool->size_its_cutting_edges ()) {
	Cutting_component *component = Cutting_component::newInstance (des);
	tool->add_its_cutting_edges (component->getRoot());
    }

    Cutting_component *component = Cutting_component::find (tool->get_its_cutting_edges (component_index)->getValue());

    Milling_technology * tech = Milling_technology::find (component->get_its_technology());
    if (tech == NULL) {
	tech = Milling_technology::newInstance (des);
	component->put_its_technology (tech->getRoot());
    }

    tech->put_spindle (stix_make_measure_item (des, spindle_speed, speed_unit));
    return tool;
}

Milling_machine_cutting_tool_IF	*set_tool_recommended_feed (Milling_machine_cutting_tool_IF *tool, double feedrate, StixUnit feed_unit, unsigned component_index)
{
    if (tool == NULL || feed_unit == NULL) {
	REPORT_ERROR ("Set tool recommended speed: function called with NULL arguments");
	return NULL;
    }
    if (component_index != 0 && component_index > tool->size_its_cutting_edges ()) {
	REPORT_ERROR ("Set tool recommended feedrate for tool %s: component index %d out of range [0, %d]", tool->get_its_id(), component_index, tool->size_its_cutting_edges () - 1);
	return NULL;
    }
    TRACE_CALL ("Recommended feedrate for tool %s component %d set to %g (%s)", tool->get_its_id (), component_index, feedrate, stix_get_unit_name (feed_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    if (component_index >=  tool->size_its_cutting_edges ()) {
	Cutting_component *component = Cutting_component::newInstance (des);
	tool->add_its_cutting_edges (component->getRoot());
    }

    Cutting_component *component = Cutting_component::find (tool->get_its_cutting_edges (component_index)->getValue());

    Milling_technology * tech = Milling_technology::find (component->get_its_technology());
    if (tech == NULL) {
	tech = Milling_technology::newInstance (des);
	component->put_its_technology (tech->getRoot());
    }

    tech->put_feedrate (stix_make_measure_item (des, feedrate, feed_unit));
    return tool;
}


// set number of cutting components - give each same data as first component
Milling_machine_cutting_tool_IF	*set_tool_number_of_flutes (Milling_machine_cutting_tool_IF *tool, unsigned flute_count)
{
    if (tool == NULL) {
	REPORT_ERROR ("Set tool number of flutes: function called with NULL arguments");
	return NULL;
    }
    if (tool->size_its_cutting_edges () < 1) {
	REPORT_ERROR ("Set number of flutes for tool %s: must have at least one flute", tool->get_its_id());
	return NULL;
    }
    TRACE_CALL ("Setting number of flutes for tool %s to %d", tool->get_its_id (), flute_count);

    RoseDesign *des = tool->getRootObject ()->design();

    Cutting_component *first = Cutting_component::find (tool->get_its_cutting_edges (0)->getValue());

    for (unsigned i = tool->size_its_cutting_edges (); i < flute_count; i++) {
	Cutting_component *next = Cutting_component::newInstance (des);
	next->put_expected_tool_life (first->get_expected_tool_life());
	next->put_its_material (first->get_its_material ());
	next->put_its_technology (first->get_its_technology ());
	next->put_tool_functional_length (first->get_tool_functional_length ());
	tool->add_its_cutting_edges (next->getRoot());
    }

    return tool;
}

Milling_machine_cutting_tool_IF	*set_tool_overall_length (Milling_machine_cutting_tool_IF *tool, double length, StixUnit length_unit)
{
    if (tool == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set tool overall length: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting overall assembly length for tool %s to %g (%s)", tool->get_its_id (), length, stix_get_unit_name (length_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    tool->put_overall_assembly_length (stix_make_measure_item (des, length, length_unit));

    return tool;
}

Milling_machine_cutting_tool_IF	*set_tool_flute_length (Milling_machine_cutting_tool_IF *tool, double length, StixUnit length_unit)
{
    if (tool == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set tool flute length: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting flute length (max depth of cut) for tool %s to %g (%s)", tool->get_its_id (), length, stix_get_unit_name (length_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    tool->put_maximum_depth_of_cut (stix_make_measure_item (des, length, length_unit));

    return tool;
}

Milling_machine_cutting_tool_IF	*set_tool_diameter (Milling_machine_cutting_tool_IF *tool, double diameter, StixUnit length_unit)
{
    if (tool == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set tool diameter: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting effective cutting diameter for tool %s to %g (%s)", tool->get_its_id (), diameter, stix_get_unit_name (length_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    tool->put_effective_cutting_diameter (stix_make_measure_item (des, diameter, length_unit));

    return tool;
}

Milling_cutting_tool_IF	*set_tool_radius (Milling_cutting_tool_IF *tool, double radius, StixUnit length_unit)
{
    if (tool == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set tool edge radius: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting edge radius for tool %s to %g (%s)", tool->get_its_id (), radius, stix_get_unit_name (length_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    tool->put_edge_radius (stix_make_measure_item (des, radius, length_unit));

    return tool;
}

Endmill	*set_tool_taper_angle (Endmill *tool, double angle, StixUnit angle_unit)
{
    if (tool == NULL || angle_unit == NULL) {
	REPORT_ERROR ("Set tool taper angle: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting tapr angle for tool %s to %g (%s)", tool->get_its_id (), angle, stix_get_unit_name (angle_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    tool->put_tool_cutting_edge_angle (stix_make_measure_item (des, angle, angle_unit));

    return tool;
}

Facemill *set_tool_taper_angle (Facemill *tool, double angle, StixUnit angle_unit)
{
    if (tool == NULL || angle_unit == NULL) {
	REPORT_ERROR ("Set tool taper angle: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting taper angle for tool %s to %g (%s)", tool->get_its_id (), angle, stix_get_unit_name (angle_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    tool->put_tool_cutting_edge_angle (stix_make_measure_item (des, angle, angle_unit));

    return tool;
}

Drilling_cutting_tool_IF *set_tool_tip_angle (Drilling_cutting_tool_IF *tool, double angle, StixUnit angle_unit)
{
    if (tool == NULL || angle_unit == NULL) {
	REPORT_ERROR ("Set tool tip angle: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting tip angle for tool %s to %g (%s)", tool->get_its_id (), angle, stix_get_unit_name (angle_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    tool->put_point_angle (stix_make_measure_item (des, angle, angle_unit));

    return tool;
}

Milling_machine_cutting_tool_IF	*set_tool_hand_of_cut (Milling_machine_cutting_tool_IF *tool,  const char* left_neutral_right)
{
    if (tool == NULL) {
	REPORT_ERROR ("Set tool hand of cut: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting hand of cut for tool %s to %s", tool->get_its_id (), left_neutral_right);

    RoseDesign *des = tool->getRootObject ()->design();

    // #define EN_LEFT "left"
    // #define EN_NEUTRAL "neutral"
    // #define EN_RIGHT "right"
    tool->put_hand_of_cut (left_neutral_right);
    return tool;
}

Milling_machine_cutting_tool_IF	*set_tool_coolant_through_tool (Milling_machine_cutting_tool_IF *tool, const char* on_off)
{
    if (tool == NULL) {
	REPORT_ERROR ("Set coolant through for tool: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting coolant through for tool %s to %s", tool->get_its_id (), on_off);

    RoseDesign *des = tool->getRootObject ()->design();

    // #define EN_THROUGH_SPINDLE_COOLANT_OFF = "through spindle coolant off"
    // #define EN_THROUGH_SPINDLE_COOLANT_ON "through spindle coolant on"

    tool->put_coolant_through_tool (on_off);
    return tool;
}

Tapping_cutting_tool *set_tool_thread_form_type (Tapping_cutting_tool *tool, const char *form_type)
{
    if (tool == NULL) {
	REPORT_ERROR ("Set tool thread form type: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting thread from type for tool %s to %s", tool->get_its_id (), form_type);

    RoseDesign *des = tool->getRootObject ()->design();

    tool->put_thread_form_type (form_type);
    return tool;
}

Tapping_cutting_tool *set_tool_thread_size (Tapping_cutting_tool *tool, double size, StixUnit length_unit)
{
    if (tool == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set tool thread size: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting thread size for tool %s to %g (%s)", tool->get_its_id (), size, stix_get_unit_name (length_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    tool->put_thread_size (stix_make_measure_item (des, size, length_unit));
    return tool;
}

Tapping_cutting_tool *set_tool_thread_pitch (Tapping_cutting_tool *tool, double pitch, StixUnit length_unit)
{
    if (tool == NULL || length_unit == NULL) {
	REPORT_ERROR ("Set tool thread pitch: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting thread pitch for tool %s to %g (%s)", tool->get_its_id (), pitch, stix_get_unit_name (length_unit));

    RoseDesign *des = tool->getRootObject ()->design();

    tool->put_thread_pitch (stix_make_measure_item (des, pitch, length_unit));
    return tool;
}

Tapping_cutting_tool *set_tool_thread_taper_count (Tapping_cutting_tool *tool, double count)
{
    if (tool == NULL) {
	REPORT_ERROR ("Set tool thread taper count: function called with NULL arguments");
	return NULL;
    }
    TRACE_CALL ("Setting thread taper count for tool %s to %g", tool->get_its_id (), count);

    RoseDesign *des = tool->getRootObject ()->design();

    tool->put_taper_thread_count (count);
    return tool;
}
